//
//  Board.m
//  WBA
//
//  Created by Class Account on 3/29/10.
//  Copyright 2010 __MyCompanyName__. All rights reserved.
//

#import "Board.h"
#import "globals.h"


@implementation Board

@synthesize sections;
@synthesize layout;
@synthesize minutes;
@synthesize physicalBoardNumber;

-(id)initWithLayoutType:(LayoutType)layType
{
	if (![super init]) return nil;
	
	self.layout = layType;
	sections = [[self setupRegions:layType] retain];
	physicalBoardNumber = -1;
	
	return self;
}


-(id)initWithDict:(NSDictionary*) dict
{
  [super init];

  self.layout = [[dict objectForKey:BOARD_LAYOUT_KEY] intValue];
  self.minutes = [[dict objectForKey:BOARD_MINS_KEY] intValue];
  self.physicalBoardNumber = [[dict objectForKey:BOARD_PHYSICAL_KEY] intValue];

  // The board sections array is initilaized from dict, and each section's region
  // values are loaded from the bsect's dict
  sections = [[NSMutableArray alloc] init];
  [sections retain];
  for (NSMutableDictionary *d in [dict objectForKey:BOARD_SECTIONS_KEY])
    {
      [d retain];
      BoardSection *bs = [[[BoardSection alloc] initWithDict:d] retain];
      [sections addObject:bs];
      [d release];
    }
	
  return self;
  
}

-(NSMutableDictionary *)serialize {
  NSMutableDictionary * boardDictionary = [[NSMutableDictionary alloc] init];
  [boardDictionary setObject:[[NSString alloc] initWithFormat:@"%d", layout] 
		   forKey:BOARD_LAYOUT_KEY];
  [boardDictionary setObject:[[NSString alloc] initWithFormat: @"%d", minutes] 
		   forKey:BOARD_MINS_KEY];
  [boardDictionary setObject:[[NSString alloc] initWithFormat:@"%d", physicalBoardNumber] 
		   forKey:BOARD_PHYSICAL_KEY];

  NSMutableArray *boardSectionsList = [[NSMutableArray alloc] init];
  for (BoardSection *section in sections) {
    [boardSectionsList addObject:[section serialize]];
  }
  [boardDictionary setObject:boardSectionsList forKey:BOARD_SECTIONS_KEY];
	
  [boardDictionary retain];
  return boardDictionary;
}


-(id)initAsCopy:(Board*)other
{
	[super init];
	
	self.layout = other.layout;
	self.minutes = other.minutes;
	self.physicalBoardNumber = other.physicalBoardNumber;
	NSMutableArray *tmp = [[NSMutableArray alloc] init];
	for (id obj in other.sections)
	{
		BoardSection *bs = [[BoardSection alloc] init];
		[bs copyFrom:obj];
		bs.region = ((BoardSection*)obj).region;
		[tmp addObject:bs];
		[bs release];
	}
	self.sections = [[NSArray alloc] initWithArray:tmp];
	
	[tmp release];
	return self;
}

-(NSArray*)setupRegions:(LayoutType)layType
{
	NSArray *secs;
	if (layType == FullBoard)
	{
		BoardSection *sec = [[BoardSection alloc] init];
		sec.region = CGRectMake(0, 0, 1, 1);
		secs = [NSArray arrayWithObject:sec];
		[secs retain];
		[sec release];
	}
	else if (layType == Horiz2Split)
	{
		BoardSection *sec1 = [[BoardSection alloc] init];
		BoardSection *sec2 = [[BoardSection alloc] init];
		sec1.region = CGRectMake(0, 0, 1, 0.5);
		sec2.region = CGRectMake(0, 0.5, 1, 0.5);
		secs = [NSArray arrayWithObjects:sec1, sec2, nil];
		[secs retain];
		[sec1 release];
		[sec2 release];
	}
	else if (layType == Vert2Split)
	{
		BoardSection *sec1 = [[BoardSection alloc] init];
		BoardSection *sec2 = [[BoardSection alloc] init];
		sec1.region = CGRectMake(0, 0, 0.5, 1);
		sec2.region = CGRectMake(0.5, 0, 0.5, 1);
		secs = [NSArray arrayWithObjects:sec1, sec2, nil];
		[secs retain];
		[sec1 release];
		[sec2 release];
	}
	else if (layType == Vert2and1Split)
	{
		BoardSection *sec1 = [[BoardSection alloc] init];
		BoardSection *sec2 = [[BoardSection alloc] init];
		BoardSection *sec3 = [[BoardSection alloc] init];
		sec1.region = CGRectMake(0, 0, 0.5, 0.5);
		sec2.region = CGRectMake(0, 0.5, 0.5, 0.5);
		sec3.region = CGRectMake(0.5, 0, 0.5, 1);
		secs = [NSArray arrayWithObjects:sec1, sec2, sec3, nil];
		[secs retain];
		[sec1 release];
		[sec2 release];
		[sec3 release];
	}
	else if (layType == Vert1and2Split)
	{
		BoardSection *sec1 = [[BoardSection alloc] init];
		BoardSection *sec2 = [[BoardSection alloc] init];
		BoardSection *sec3 = [[BoardSection alloc] init];
		sec1.region = CGRectMake(0, 0, 0.5, 1);
		sec2.region = CGRectMake(0.5, 0, 0.5, 0.5);
		sec3.region = CGRectMake(0.5, 0.5, 0.5, 0.5);
		secs = [NSArray arrayWithObjects:sec1, sec2, sec3, nil];
		[secs retain];
		[sec1 release];
		[sec2 release];
		[sec3 release];
	}
	else if (layType == Vert3Split)
	{
		BoardSection *sec1 = [[BoardSection alloc] init];
		BoardSection *sec2 = [[BoardSection alloc] init];
		BoardSection *sec3 = [[BoardSection alloc] init];
		sec1.region = CGRectMake(0, 0, 1.0/3, 1);
		sec2.region = CGRectMake(1.0/3, 0, 1.0/3, 1);
		sec3.region = CGRectMake(2.0/3, 0, 1.0/3, 1);
		secs = [NSArray arrayWithObjects:sec1, sec2, sec3, nil];
		[secs retain];
		[sec1 release];
		[sec2 release];
		[sec3 release];
	}
	else if (layType == Horiz2and1Split)
	{
		BoardSection *sec1 = [[BoardSection alloc] init];
		BoardSection *sec2 = [[BoardSection alloc] init];
		BoardSection *sec3 = [[BoardSection alloc] init];
		sec1.region = CGRectMake(0, 0, 0.5, 0.5);
		sec2.region = CGRectMake(0.5, 0, 0.5, 0.5);
		sec3.region = CGRectMake(0, 0.5, 1, 0.5);
		secs = [NSArray arrayWithObjects:sec1, sec2, sec3, nil];
		[secs retain];
		[sec1 release];
		[sec2 release];
		[sec3 release];
	}
	else if (layType == Horiz1and2Split)
	{
		BoardSection *sec1 = [[BoardSection alloc] init];
		BoardSection *sec2 = [[BoardSection alloc] init];
		BoardSection *sec3 = [[BoardSection alloc] init];
		sec1.region = CGRectMake(0, 0, 1, 0.5);
		sec2.region = CGRectMake(0, 0.5, 0.5, 0.5);
		sec3.region = CGRectMake(0.5, 0.5, 0.5, 0.5);
		secs = [NSArray arrayWithObjects:sec1, sec2, sec3, nil];
		[secs retain];
		[sec1 release];
		[sec2 release];
		[sec3 release];
	}
	else if (layType == Horiz3Split)
	{
		BoardSection *sec1 = [[BoardSection alloc] init];
		BoardSection *sec2 = [[BoardSection alloc] init];
		BoardSection *sec3 = [[BoardSection alloc] init];
		sec1.region = CGRectMake(0, 0, 1, 1.0/3);
		sec2.region = CGRectMake(0, 1.0/3, 1, 1.0/3);
		sec3.region = CGRectMake(0, 2.0/3, 1, 1.0/3);
		secs = [NSArray arrayWithObjects:sec1, sec2, sec3, nil];
		[secs retain];
		[sec1 release];
		[sec2 release];
		[sec3 release];
	}
	else if (layType = FourWaySplit)
	{
		BoardSection *sec1 = [[BoardSection alloc] init];
		BoardSection *sec2 = [[BoardSection alloc] init];
		BoardSection *sec3 = [[BoardSection alloc] init];
		BoardSection *sec4 = [[BoardSection alloc] init];
		sec1.region = CGRectMake(0, 0, 0.5, 0.5);
		sec2.region = CGRectMake(0, 0.5, 0.5, 0.5);
		sec3.region = CGRectMake(0.5, 0, 0.5, 0.5);
		sec4.region = CGRectMake(0.5, 0.5, 0.5, 0.5);
		secs = [NSArray arrayWithObjects:sec1, sec2, sec3, sec4, nil];
		[secs retain];
		[sec1 release];
		[sec2 release];
		[sec3 release];
		[sec4 release];
	}
	else
	{
		secs = nil;
	}
	
	return secs;
}

-(void)changeLayout:(LayoutType)layType
{
	layout = layType;
	NSArray *newSecs = [self setupRegions:layType];
	NSMutableArray *filledOldSecs = [[NSMutableArray alloc] init];
	
	// Fill up filledOldSecs with old sections that weren't empty.
	for (id obj in sections)
	{
		if ([obj type] != Empty)
			[filledOldSecs addObject:obj];
	}
	
	int numsecs = MIN(newSecs.count, filledOldSecs.count);
	for (int i = 0; i < numsecs; i++)
	{
		[[newSecs objectAtIndex:i] copyFrom:[filledOldSecs objectAtIndex:i]];
	}
	
	[filledOldSecs release];
	[sections release];
	sections = newSecs;
	[newSecs release];
	[sections retain];
}

-(BOOL)isBlank
{
	for (id obj in self.sections)
	{
		if ([obj type] != Empty)
			return NO;
	}
	return YES;
}

-(BoardSection*)sectionContainingPoint:(CGPoint)point
{
	for (id obj in self.sections)
	{
		if ([obj containsPoint:point])
			return obj;
	}
	
	// If no section contains the point
	return nil;
}

-(bool)pointOnPhotoButton:(CGPoint)point atSection:(BoardSection *) section 
		   region:(CGRect) drawRegion thumbSize:(int) thumbSize
{
  return [section pointOnPhotoButton:point  region:drawRegion thumbSize:thumbSize];
}

-(bool)pointOnFlagButton:(CGPoint)point atSection:(BoardSection *) section region:(CGRect) drawRegion
{
  return [section pointOnFlagButton:point  region:drawRegion];
}


-(void)dealloc
{
	if (sections)
		[sections release];
	[super dealloc];
}

@end
