//
//  BoardSection.m
//  WBA
//
//  Created by Class Account on 3/29/10.
//  Copyright 2010 __MyCompanyName__. All rights reserved.
//

#import "BoardSection.h"


@implementation BoardSection

@synthesize type;
@synthesize region;
@synthesize hasText;
@synthesize hasPhoto;
@synthesize soundFile;
@synthesize textAnnotation;

-(id) init
{
  [super init];
  self.type = Empty;
  hasText  = NO;
  hasPhoto = NO;
  soundFile = nil;
  
  return self;
}


-(id) initWithDict:(NSDictionary*) dict
{
  [super init];

  // Defaults
  self.type = Empty;
  hasText  = NO;
  hasPhoto = NO;
  soundFile = nil;

  // Setup from dict
  type = [[dict objectForKey:BSECT_ELEEMENT_KEY] intValue];

  NSString *x =  [dict objectForKey:BSECT_REGION_X];
  NSString *y =  [dict objectForKey:BSECT_REGION_Y];
  NSString *w =  [dict objectForKey:BSECT_REGION_W];
  NSString *h =  [dict objectForKey:BSECT_REGION_H];

  
  region = CGRectMake([x floatValue], 
		      [y floatValue], 
		      [w floatValue], 
		      [h floatValue]);

  NSString *loaded_textAnnotation = [dict objectForKey:BSECT_TEXT_ANNOTATION_KEY];
  if (loaded_textAnnotation) {
    textAnnotation = loaded_textAnnotation;
    hasText = YES;
    [textAnnotation retain];
  }
  NSString *loaded_soundFile = [dict objectForKey:BSECT_SOUND_FILE_KEY];
  if (loaded_soundFile) {
    soundFile = loaded_soundFile;
    [soundFile retain];
  }
  NSString *loaded_photoFile = [dict objectForKey:BSECT_PHOTO_FILE_KEY];
  if (loaded_photoFile) {
    photoFile = loaded_photoFile;
    photoAnnotation = [UIImage imageWithContentsOfFile:loaded_photoFile];
    [photoFile retain];
    [photoAnnotation retain];
    hasPhoto = YES;
  }
	
  return self;
}

-(NSMutableDictionary *) serialize {
  NSMutableDictionary *boardSectionDictionary = [[NSMutableDictionary alloc] init];
	
  [boardSectionDictionary setObject:[[NSString alloc] initWithFormat:@"%d", type] 
			  forKey:BSECT_ELEEMENT_KEY];

  [boardSectionDictionary setObject:[[NSString alloc] initWithFormat:@"%f", region.origin.x] 
			  forKey:BSECT_REGION_X];
  [boardSectionDictionary setObject:[[NSString alloc] initWithFormat:@"%f", region.origin.y] 
			  forKey:BSECT_REGION_Y];
  [boardSectionDictionary setObject:[[NSString alloc] initWithFormat:@"%f", region.size.width] 
			  forKey:BSECT_REGION_W];
  [boardSectionDictionary setObject:[[NSString alloc] initWithFormat:@"%f", region.size.height] 
			  forKey:BSECT_REGION_H];


  if ([self hasText]) {
    [boardSectionDictionary setObject:textAnnotation forKey:BSECT_TEXT_ANNOTATION_KEY];
  }
  if ([self hasPhoto]) {
    [boardSectionDictionary setObject:[self getPhotoFilePath] forKey:BSECT_PHOTO_FILE_KEY];
  }
  if ([self hasFlag]) {
    [boardSectionDictionary setObject:soundFile forKey:BSECT_SOUND_FILE_KEY];
  }
  [boardSectionDictionary retain];
	
  return boardSectionDictionary;
}


// Does NOT copy region
-(void)copyFrom:(BoardSection*)other
{	
	type = other.type;
	hasText = other.hasText;
	hasPhoto = other.hasPhoto;
	soundFile = other.soundFile;
	[soundFile retain];
	if (hasText)
	{
		if (textAnnotation)
			[textAnnotation release];
		textAnnotation = [[NSString alloc] initWithString:other.textAnnotation];
	}
	if (hasPhoto)
	{
	  photoAnnotation = [other getPhotoAnnotation];
	  [photoAnnotation retain];
	}
}

-(bool)containsPoint:(CGPoint)point
{
	return (point.x >= region.origin.x && point.x < region.origin.x+region.size.width &&
	        point.y >= region.origin.y && point.y < region.origin.y+region.size.height);
}


-(bool)pointOnPhotoButton:(CGPoint)point region:(CGRect)drawRegion thumbSize:(int) thumbSize
{
  float x, y, w, h;
  CGSize imageDim = [photoAnnotation size];
  int separatorWidth = 2;
  /* The thumbnail must fit in a square that is 
   * no more than thumbSize x thumbSize pixels, and the thumbnail
   * should maintain its aspect ratio.
   */
  w = ((imageDim.width > imageDim.height) ? 
       thumbSize : thumbSize * (imageDim.width / imageDim.height)) / drawRegion.size.width;
  h = ((imageDim.width > imageDim.height) ? 
       thumbSize * (imageDim.height / imageDim.width) : thumbSize) / drawRegion.size.height;
  
  x = (drawRegion.origin.x + [self getOriginX]*drawRegion.size.width + 
       separatorWidth + PHOTO_THUMB_PADDING) / drawRegion.size.width;
  y = (drawRegion.origin.y + [self getOriginY]*drawRegion.size.height + 
    [self getHeight]*drawRegion.size.height -
       h - separatorWidth - PHOTO_THUMB_PADDING) / drawRegion.size.height;

  if (hasPhoto && point.x >= x && point.x < (x+w) &&
      point.y >= (y-h) && point.y < y) {
    NSLog(@"Point is on photo button\n");
  }
  return (hasPhoto && point.x >= x && point.x < (x+w) &&
	  point.y >= (y-h) && point.y < y);
  
}

-(bool)pointOnFlagButton:(CGPoint)point  region:(CGRect)drawRegion
{
  float normButtonDimX = BUTTON_DIM_X / drawRegion.size.width;
  float normButtonDimY = BUTTON_DIM_Y / drawRegion.size.height;

  return (point.x > region.origin.x &&
	  point.x <= region.origin.x+normButtonDimX &&
	  point.y > region.origin.y &&
	  point.y <= region.origin.y+normButtonDimY);

}


-(void)processEditorTap:(ElementType)elType withViewController:(UIViewController*)vc
{
	if (elType == Empty && type != Empty) {
		/* Bring up annotations menu */
		[vc setAnnotation:self];
	} else {
		/* Tapped with a ElementType selected. "Tag" this board section
		* with elType
		*/
		type = elType;
	}
}

-(void)clearContents
{
	hasPhoto = NO;
	hasText = NO;
	type = Empty;
	[textAnnotation release];
	textAnnotation = nil;
	[photoAnnotation release];
	photoAnnotation = nil;
}

-(void)setPhotoAnnotation:(UIImage*) image
{
	hasPhoto = YES;
	photoAnnotation = image;
	[photoAnnotation retain];
}
-(UIImage*)getPhotoAnnotation
{
	return photoAnnotation;
}
-(void)removePhotoAnnotation
{
  if (hasPhoto)
  {
	hasPhoto = NO;
	[photoAnnotation release];
	photoAnnotation = nil;
  }
}
-(void)setTextAnnotation:(NSString*) s
{
  hasText = YES;
  textAnnotation = s;
  [textAnnotation retain];
}

-(NSString*)getTextAnnotation
{
	return textAnnotation;
}
-(void)removeTextAnnotation
{
  if (hasText)
  {
	hasText = NO;
	[textAnnotation release];
	textAnnotation = nil;
  }
}

-(void)setFlag:(NSString*) soundPath
{
  if ([self hasFlag]) {
    NSLog(@"Removing old sound file before setting new sound file %@\n", soundPath);
    [soundPath release];
    [self removeFlag];
  }
  NSLog(@"the new sound file path is %@\n", soundPath);
  soundFile = soundPath;
  [soundFile retain];
}
-(void)removeFlag
{
  if ([self hasFlag]) {
    NSFileManager *fileManager = [NSFileManager defaultManager];
    [fileManager removeItemAtPath:soundFile error:NULL];
    [soundFile release];
    soundFile = nil;
  }
}
-(void)setPhotoFilePath:(NSString*) path
{
	if (photoFile != nil) {
		NSFileManager *fileManager = [NSFileManager defaultManager];
		[fileManager removeItemAtPath:photoFile error:NULL];
	}
	photoFile = path;
	[photoFile retain];
}
-(NSString*)getPhotoFilePath
{
	return photoFile;
}

-(float) getOriginX { return self.region.origin.x;    }
-(float) getOriginY { return self.region.origin.y;    }
-(float) getWidth   { return self.region.size.width;  }
-(float) getHeight  { return self.region.size.height; }
-(bool)  hasPhoto   { return hasPhoto; }
-(bool)  hasText    { return hasText;  }
-(bool)  hasFlag    { return soundFile != nil;  }

-(void)dealloc
{
	if (textAnnotation)
	  [textAnnotation release];
	[super dealloc];
}

@end
