//
//  BoardView.m
//  WBA
//
//  Created by Class Account on 4/2/10.
//  Copyright 2010 __MyCompanyName__. All rights reserved.
//

#import "BoardView.h"


@implementation BoardView

@synthesize model;

-(void)commonConstructor
{
	myLabels = [[NSMutableArray alloc] init];
	drawRegion = CGRectMake(0, 0, 100, 100);
	separatorWidth = 2;
	flagSize = 20;
	thumbSize = 60;
}

- (id)initWithFrame:(CGRect)frame {
    if (self = [super initWithFrame:frame])
	{
        [self commonConstructor];
    }
    return self;
}

-(id)initWithCoder:(NSCoder*)coder
{
	if (self = [super initWithCoder:coder])
	{
		[self commonConstructor];
	}
	return self;
}

-(void)setModel:(Board*)mod
{
	if (model)
		[model release];
	model = mod;
	[model retain];
	
	[self setNeedsDisplay];
}

-(void)setViewController:(UIViewController*)vc
{
	if (controller)
		[controller release];
	controller = vc;
	[controller retain];
}

-(BoardSection*)sectionContainingPoint:(CGPoint)point
{
	// Convert to percentage coordinate system.
	point.x -= drawRegion.origin.x;
	point.y -= drawRegion.origin.y;
	point.x /= drawRegion.size.width;
	point.y /= drawRegion.size.height;
	return [model sectionContainingPoint:point];
}

-(bool)pointOnPhotoButton:(CGPoint)point 
{
	BoardSection *sec = [self sectionContainingPoint:point];
	point.x -= drawRegion.origin.x;
	point.y -= drawRegion.origin.y;
	point.x /= drawRegion.size.width;
	point.y /= drawRegion.size.height;	
	return [model pointOnPhotoButton:point atSection:sec region:drawRegion thumbSize:thumbSize];
}

-(bool)pointOnFlagButton:(CGPoint)point 
{
	BoardSection *sec = [self sectionContainingPoint:point];
	point.x -= drawRegion.origin.x;
	point.y -= drawRegion.origin.y;
	point.x /= drawRegion.size.width;
	point.y /= drawRegion.size.height;	
	return [model pointOnFlagButton:point atSection:sec region:drawRegion];
}

- (void)dealloc {
	if (model)
		[model release];
	if (controller)
		[controller release];
	if (myLabels)
		[myLabels release];
    [super dealloc];
}


#pragma mark drawing

- (void)drawRect:(CGRect)rect {
	
    // Drawing code
	CGContextRef contextRef = UIGraphicsGetCurrentContext();
	
	//Doesn't work vv
	CGContextSetFillColorWithColor(contextRef, [UIColor lightGrayColor].CGColor);
	CGContextFillRect(contextRef, CGRectMake(drawRegion.origin.x, drawRegion.origin.y, drawRegion.size.width, drawRegion.size.height));
	
	// Draw the whiteboard background
	if (model.sections != nil){
		
		CGContextSetRGBFillColor(contextRef, 1, 1, 1, 1);
		CGContextFillRect(contextRef, CGRectMake(drawRegion.origin.x, drawRegion.origin.y, drawRegion.size.width, drawRegion.size.height));
	
		// Draw the actual board contents
		[self drawBoard];
	}
}

-(void)drawBoard
{
	// Remove labels from last draw session.
	for (id aLabel in myLabels)
	{
		UILabel *l = (UILabel*) aLabel;
		[l removeFromSuperview];
	}
	[myLabels removeAllObjects];
	
	// Actually draw board contents
	for (id obj in model.sections)
	{
		BoardSection *sec = (BoardSection*)obj;
		[self drawSection:sec];
	}
	
	// Draw separating lines around sections
	[self drawSectionOutlines];
	
	//[self setNeedsDisplay];
}

-(void)drawSectionOutlines
{
	// Always draw an outline around the board
	CGContextRef contextRef = UIGraphicsGetCurrentContext();
	CGContextSetRGBFillColor(contextRef, 0, 0, 0, 1);
	CGContextFillRect(contextRef, CGRectMake(drawRegion.origin.x, drawRegion.origin.y, separatorWidth, drawRegion.size.height));
	CGContextFillRect(contextRef, CGRectMake(drawRegion.origin.x+drawRegion.size.width-separatorWidth, drawRegion.origin.y, separatorWidth, drawRegion.size.height));
	CGContextFillRect(contextRef, CGRectMake(drawRegion.origin.x, drawRegion.origin.y, drawRegion.size.width, separatorWidth));
	CGContextFillRect(contextRef, CGRectMake(drawRegion.origin.x, drawRegion.origin.y+drawRegion.size.height-separatorWidth, drawRegion.size.width, separatorWidth));
	
	// Add stuff that depends upon the layoutType
	LayoutType ltype = model.layout;
	if (ltype == Vert2Split | ltype == Vert1and2Split || ltype == Vert2and1Split || ltype == FourWaySplit)
	{
		// Vertical line down the middle
		CGContextFillRect(contextRef, CGRectMake(drawRegion.origin.x+drawRegion.size.width/2-separatorWidth/2, drawRegion.origin.y, separatorWidth, drawRegion.size.height));
	}
	if (ltype == Horiz2Split || ltype == Horiz2and1Split || ltype == Horiz1and2Split || ltype == FourWaySplit)
	{
		// Horizontal line down the middle
		CGContextFillRect(contextRef, CGRectMake(drawRegion.origin.x, drawRegion.origin.y+drawRegion.size.height/2-separatorWidth/2, drawRegion.size.width, separatorWidth));
	}
	if (ltype == Vert2and1Split)
	{
		// Horizontal first-half line down the middle
		CGContextFillRect(contextRef, CGRectMake(drawRegion.origin.x, drawRegion.origin.y+drawRegion.size.height/2-separatorWidth/2, drawRegion.size.width/2, separatorWidth));
	}
	if (ltype == Vert1and2Split)
	{
		// Horizontal second-half line down the middle.
		CGContextFillRect(contextRef, CGRectMake(drawRegion.origin.x+drawRegion.size.width/2-separatorWidth/2, drawRegion.origin.y+drawRegion.size.height/2-separatorWidth/2, drawRegion.size.width/2, separatorWidth));
	}
	if (ltype == Horiz2and1Split)
	{
		// Vertical first-half line down the middle
		CGContextFillRect(contextRef, CGRectMake(drawRegion.origin.x+drawRegion.size.width/2-separatorWidth/2, drawRegion.origin.y, separatorWidth, drawRegion.size.height/2));
	}
	if (ltype == Horiz1and2Split)
	{
		// Vertical second-half line down the middle.
		CGContextFillRect(contextRef, CGRectMake(drawRegion.origin.x+drawRegion.size.width/2-separatorWidth/2, drawRegion.origin.y+drawRegion.size.height/2-separatorWidth/2, separatorWidth, drawRegion.size.height/2));
	}
	if (ltype == Vert3Split)
	{
		// Vertical line down the 1/3 mark
		CGContextFillRect(contextRef, CGRectMake(drawRegion.origin.x+drawRegion.size.width/3-separatorWidth/2, drawRegion.origin.y, separatorWidth, drawRegion.size.height));
	}
	if (ltype == Vert3Split)
	{
		// Vertical line down the 2/3 mark
		CGContextFillRect(contextRef, CGRectMake(drawRegion.origin.x+2*drawRegion.size.width/3-separatorWidth/2, drawRegion.origin.y, separatorWidth, drawRegion.size.height));
	}
	if (ltype == Horiz3Split)
	{
		// Horiz line down the 1/3 mark
		CGContextFillRect(contextRef, CGRectMake(drawRegion.origin.x, drawRegion.origin.y+drawRegion.size.height/3-separatorWidth/2, drawRegion.size.width, separatorWidth));
	}
	if (ltype == Horiz3Split)
	{
		// Horiz line down the 2/3 mark
		CGContextFillRect(contextRef, CGRectMake(drawRegion.origin.x, drawRegion.origin.y+2*drawRegion.size.height/3-separatorWidth/2, drawRegion.size.width, separatorWidth));
	}
}

-(void)drawBoardHelpNotice
{
	UILabel *myLabel = [[UILabel alloc] 
						initWithFrame:CGRectMake(drawRegion.origin.x+20,
												 drawRegion.origin.y+20,
												 drawRegion.size.width-40,
												 drawRegion.size.height-40)];
	myLabel.text = @"Drag elements from the right onto the board to begin.";
	myLabel.numberOfLines = 0;
	myLabel.backgroundColor = [UIColor clearColor];
	myLabel.textAlignment = UITextAlignmentCenter;
	[self addSubview:myLabel];
	[myLabels addObject:myLabel];
	[myLabel release];
}

-(void)drawSection:(BoardSection*)section
{
	/* Draw content rect for the section */
	
	// Decide background image based on element type.
	if (section.type != Empty)
	{
		NSString *bgImName;
		switch (section.type)
		{
			case Proof:
			case Theorem:
			case Definition:
				bgImName = @"purple.png";
				break;
			case UOList:
			case OList:
				bgImName = @"blue.png";
				break;
			case Graph:
			case Diagram:
				bgImName = @"teal.png";
				break;
			case Example:
			case QandA:
				bgImName = @"green.png";
				break;
			case Custom:
				bgImName = @"gray.png";
				break;
			default:
				break;
				
		}
		
		UIImage *bgIm = [UIImage imageNamed:bgImName];
		[bgIm drawInRect: CGRectMake(drawRegion.origin.x + [section getOriginX]*drawRegion.size.width,
								   drawRegion.origin.y + [section getOriginY]*drawRegion.size.height, 
								   [section getWidth]*drawRegion.size.width, 
								   [section getHeight]*drawRegion.size.height)
			   blendMode:kCGBlendModeNormal alpha:0.6];
		
		// Decide foreground image based on element type
		NSString *fgImName;
		switch (section.type)
		{
			case Proof:
				fgImName = @"proof.png";
				break;
			case Theorem:
				fgImName = @"theorem.png";
				break;
			case Definition:
				fgImName = @"def.png";
				break;
			case UOList:
				fgImName = @"uolist.png";
				break;
			case OList:
				fgImName = @"olist.png";
				break;
			case Graph:
				fgImName = @"graph.png";
				break;
			case Diagram:
				fgImName = @"diagram.png";
				break;
			case Example:
				fgImName = @"example.png";
				break;
			case QandA:
				fgImName = @"qa.png";
				break;
			case Custom:
				fgImName = @"";
			default:
				break;
		}
		
		UIImage *fgIm = [UIImage imageNamed:fgImName];
		
		// Figure out aspect ratio to keep image from stretching.
		float width;
		float height;
		float originX = drawRegion.origin.x + [section getOriginX]*drawRegion.size.width;
		float originY = drawRegion.origin.y + [section getOriginY]*drawRegion.size.height;
		float secwidth = [section getWidth]*drawRegion.size.width;
		float secheight = [section getHeight]*drawRegion.size.height;
		if (secwidth > secheight)
		{
			height = secheight;
			width = secheight * fgIm.size.width/fgIm.size.height;
			originX += (secwidth - width)/2;
		}
		else
		{
			width = secwidth;
			height = secwidth * fgIm.size.height/fgIm.size.width;
			originY += (secheight - height)/2;
		}
		
		[fgIm drawInRect: CGRectMake(originX,
									 originY, 
									 width, 
									 height)
			   blendMode:kCGBlendModeNormal alpha:0.6];
	}
	
	/* Draw other contents */
	/* Enable these as necessary in subclasses */
	
//	// Draw photo icon (if present)
//	[self drawSectionPhotoIcon:section];
//	
//	// Draw text label
//	[self drawSectionText:section];
	
	
}

-(void)drawSectionPhotoIcon:(BoardSection*)section
{
	if ([section hasPhoto])
	{
	  UIImage *photoAnnotation;
	  float x, y, w, h;
	  CGSize imageDim;
	  CGRect imageFrame;
	  CGContextRef context;
	  
	  photoAnnotation = [section getPhotoAnnotation];
	  imageDim = [photoAnnotation size];
	  
	  /* The thumbnail must fit in a square that is 
	   * no more than thumbSize x thumbSize pixels, and the thumbnail
	   * should maintain its aspect ratio.
	   */
	  w = (imageDim.width > imageDim.height) ? 
	    thumbSize : thumbSize * (imageDim.width / imageDim.height);
	  h = (imageDim.width > imageDim.height) ? 
	    thumbSize * (imageDim.height / imageDim.width) : thumbSize;
	  
	  NSLog(@"image width %f, height %f\n", w, h);

	  x = drawRegion.origin.x + [section getOriginX]*drawRegion.size.width + 
	    separatorWidth + PHOTO_THUMB_PADDING;
	  y = drawRegion.origin.y + [section getOriginY]*drawRegion.size.height + 
	    [section getHeight]*drawRegion.size.height -
	    h - separatorWidth - PHOTO_THUMB_PADDING;

		
	  printf("x %f, y %f, w %f, h %f\n", x, y, w, h);
	  imageFrame = CGRectMake(x, y, w, h);
	  
	  if (context = UIGraphicsGetCurrentContext()) {
	    CGContextSetRGBStrokeColor(context, 0, 0, 0, .6);
	    CGContextStrokeRectWithWidth(context, imageFrame, PHOTO_THUMB_STROKE_WIDTH);
	  }
		
		[photoAnnotation drawInRect:imageFrame blendMode:kCGBlendModeNormal alpha: 0.7];
	}
}

-(void)drawSectionFlagIcon:(BoardSection*)section
{
	if (section.type == Empty) return;
  UIImage *flagIcon;
  if ([section hasFlag]) {
    flagIcon = [UIImage imageNamed:@"flag_on.png"];
  } else {
    flagIcon = [UIImage imageNamed:@"flag_off.png"];
  }
  [flagIcon drawInRect:CGRectMake(drawRegion.origin.x + [section getOriginX]*drawRegion.size.width+separatorWidth,
				  drawRegion.origin.y + [section getOriginY]*drawRegion.size.height+separatorWidth,
				  flagSize, flagSize)];
}

-(void)drawSectionFlagIcon:(BoardSection*)section flagCode:(int)flagCode
{
	if (section.type == Empty) return;
	UIImage *flagIcon;
	
	if (flagCode == 0) { // Use only a flag in LectureOverview  
		if ([section hasFlag]) {
			flagIcon = [UIImage imageNamed:@"flag_on.png"];
		} else {
			flagIcon = [UIImage imageNamed:@"flag_off.png"];
		}
	} else if (flagCode == 1) { // Use a flag and speaker in EditorBoardView
		if ([section hasFlag]) {
			flagIcon = [UIImage imageNamed:@"flag_speaker.png"];
		}
	} else { // Use a flag and microphone in PresentationBoardView
		if ([section hasFlag]) {
			flagIcon = [UIImage imageNamed:@"flag_mic_on.png"];
		} else {
			flagIcon = [UIImage imageNamed:@"flag_mic_off.png"];
		}		
	}

		
	[flagIcon drawInRect:CGRectMake(drawRegion.origin.x + [section getOriginX]*drawRegion.size.width+separatorWidth,
									drawRegion.origin.y + [section getOriginY]*drawRegion.size.height+separatorWidth,
									flagSize, flagSize)];
}

-(void)drawSectionText:(BoardSection*)section labelsOnly:(BOOL)lablesOnly
{
	UILabel *myLabel = [[UILabel alloc] 
						initWithFrame:CGRectMake(drawRegion.origin.x + [section getOriginX]*drawRegion.size.width,
												 drawRegion.origin.y + [section getOriginY]*drawRegion.size.height,
												 [section getWidth]*drawRegion.size.width,
												 [section getHeight]*drawRegion.size.height)];
	
	if (section.type == Empty && !lablesOnly)
		myLabel.text = @"Drag an Element to here";
	else if ([section hasText])
		myLabel.text = [section getTextAnnotation];
	else if (!lablesOnly)
		myLabel.text = @"Tap to Annotate";
	
	myLabel.numberOfLines = 0;
	myLabel.backgroundColor = [UIColor clearColor];
	myLabel.textAlignment = UITextAlignmentCenter;
	
	[self addSubview:myLabel];
	[myLabels addObject:myLabel];
	[myLabel release];
}



@end
