//
//  BoardViewController.m
//  WBA
//
//  Created by Class Account on 3/18/10.
//  Copyright 2010 __MyCompanyName__. All rights reserved.
//

#import "BoardViewController.h"
#import "EditorBoardView.h"
#import "LectureOverviewViewController.h"
#import "HelpViewController.h"
#import "ChooseLayoutViewController.h"
#import "TimeSetViewController.h"
#import "PhysBoardLocViewController.h"


@implementation BoardViewController

@synthesize scrollElementsView, scrollBoardsView;
@synthesize myView, overlay;
@synthesize myToolbar;
@synthesize changeLayout;
@synthesize accept;
@synthesize add;
@synthesize boardPos;
@synthesize lecture;
@synthesize positionInLecture;

BOOL isNewBoard;

 // The designated initializer.  Override if you create the controller programmatically and want to perform customization that is not appropriate for viewDidLoad.
- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
    if (self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil]) {
        // Custom initialization
		
		/* Set up logic */
		hasSelectedButton = NO;
		selectedButton = nil;
		revertImage = nil;
		
		boardViews = [[NSMutableArray alloc] init];
    }
    return self;
}

-(void)viewWillAppear:(BOOL)animated
{
	[super viewWillAppear:animated];
	
	self.navigationController.navigationBarHidden = YES;
	overlay.hidden = YES;
	overlay.delegate = self;
	[[self getBoardView] setNeedsDisplay];
	
	// Should we enable the board pos button?
	if (lecture.physBoardPos.count == 0)
		boardPos.enabled = NO;
	
	// Should we enable the timing button?
	if (!lecture.timing)
		timing.enabled = NO;
}

-(void)viewDidAppear:(BOOL)animated
{
	[super viewDidAppear:animated];
	[scrollBoardsView flashScrollIndicators];
}

// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad {
	
	// If lecture hasn't been set yet, then we're not ready to load yet
	if (!lecture)
		return;
	
    [super viewDidLoad];
	self.navigationController.navigationBarHidden = YES;
	
	/* Create the overlay */
	CGRect overlayframe = CGRectMake(0, 0, 480, 320);
	overlay = [[BoardViewOverlay alloc] initWithFrame:overlayframe];
	overlay.opaque = NO;
	[self.view addSubview:overlay];
	[self.view bringSubviewToFront:overlay];
	
	/* Set up UI elements */
	proofButton = [[self setupElementButton:@"Proof" atPosition:0] retain];
	theoremButton = [[self setupElementButton:@"Theorem" atPosition:1] retain];
	definitionButton = [[self setupElementButton:@"Definition" atPosition:2] retain];
	graphButton = [[self setupElementButton:@"Graph" atPosition:3] retain];
	diagramButton = [[self setupElementButton:@"Diagram" atPosition:4] retain];
	uoListButton = [[self setupElementButton:@"Unordered List" atPosition:5] retain];
	oListButton = [[self setupElementButton:@"Ordered List" atPosition:6] retain];
	exampleButton = [[self setupElementButton:@"Example" atPosition:7] retain];
	qaButton = [[self setupElementButton:@"Questions and Answers" atPosition:8] retain];
	customButton = [[self setupElementButton:@"Custom" atPosition:9] retain];
	
	[scrollElementsView addSubview:proofButton];
	[scrollElementsView addSubview:theoremButton];
	[scrollElementsView addSubview:definitionButton];
	[scrollElementsView addSubview:graphButton];
	[scrollElementsView addSubview:diagramButton];
	[scrollElementsView addSubview:uoListButton];
	[scrollElementsView addSubview:oListButton];
	[scrollElementsView addSubview:exampleButton];
	[scrollElementsView addSubview:qaButton];
	[scrollElementsView addSubview:customButton];
	[scrollElementsView setContentSize:CGSizeMake(100, 800)];
	
	[self setupBoardViews];
	
	// Check if we need to add the timing button
	if (!timing)
	{
		timing = [[UIBarButtonItem alloc] initWithTitle:@"Timing" style:UIBarButtonItemStyleBordered target:self action:@selector(toolbarButtonPressed:)];
		UIBarButtonItem *space = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemFlexibleSpace target:self action:nil];
		NSMutableArray *tmp = [[NSMutableArray alloc] initWithArray:myToolbar.items];
		[tmp insertObject:space atIndex:1];
		[tmp insertObject:timing atIndex:2];
		[myToolbar setItems:tmp animated:NO];
		[tmp release];
		[space release];
	}
}

- (void)viewDidUnload {
	// Release any retained subviews of the main view.
	// e.g. self.myOutlet = nil;
	[boardViews removeAllObjects];
	[boardViews release];
	[overlay release];
	[proofButton release];
	[theoremButton release];
	[definitionButton release];
	[graphButton release];
	[diagramButton release];
	[uoListButton release];
	[oListButton release];
	[exampleButton release];
	[qaButton release];
	[customButton release];
}

// Assumes that the model has been set correctly
-(void)setupBoardViews
{	
	[self refreshBoardViews];
}

-(void)refreshBoardViews
{
	for (id obj in boardViews)
		[obj removeFromSuperview];
	[boardViews removeAllObjects];
	for (int i = 0; i < [lecture numBoards]; i++)
	{
		CGRect frame = CGRectMake(EDMODE_BOARD_X+EDMODE_BOARD_WIDTH*i+2*EDMODE_BOARD_X*i, EDMODE_BOARD_Y, EDMODE_BOARD_WIDTH, EDMODE_BOARD_HEIGHT);
		EditorBoardView *ebv = [[EditorBoardView alloc] initWithFrame:frame];
		[ebv setViewController:self];
		[boardViews addObject:ebv];
		[scrollBoardsView addSubview:ebv];
		[ebv setModel:[lecture getBoard:i]];
	}
	
	[scrollBoardsView setContentSize:CGSizeMake([lecture numBoards]*scrollBoardsView.frame.size.width, scrollBoardsView.frame.size.height)];
	[scrollBoardsView setContentOffset:CGPointMake(positionInLecture*scrollBoardsView.frame.size.width,0) animated:NO];
}

-(void)addBoardToBoardViews:(int)pos
{
	for (int i = 0; i < [lecture numBoards]; i++)
	{
		// Skip all the board views that come before pos
		if (i < pos)
			continue;
		
		// Then, add a new one at pos
		else if (i == pos)
		{
			CGRect frame = CGRectMake(EDMODE_BOARD_X+EDMODE_BOARD_WIDTH*i+2*EDMODE_BOARD_X*i, EDMODE_BOARD_Y, EDMODE_BOARD_WIDTH, EDMODE_BOARD_HEIGHT);
			EditorBoardView *ebv = [[EditorBoardView alloc] initWithFrame:frame];
			[ebv setViewController:self];
			if (pos == [lecture numBoards])
				[boardViews addObject:ebv];
			else [boardViews insertObject:ebv atIndex:pos];
			[scrollBoardsView addSubview:ebv];
			[ebv setModel:[lecture getBoard:i]];
		}
		
		// Finally, adjust the frames of all the subsequent boards
		else
		{
			((EditorBoardView*)[boardViews objectAtIndex:i]).frame = CGRectMake(EDMODE_BOARD_X+EDMODE_BOARD_WIDTH*i+2*EDMODE_BOARD_X*i, EDMODE_BOARD_Y, EDMODE_BOARD_WIDTH, EDMODE_BOARD_HEIGHT);
		}
	}
	
	// in the end, adjust scroll view size
	[scrollBoardsView setContentSize:CGSizeMake([lecture numBoards]*scrollBoardsView.frame.size.width, scrollBoardsView.frame.size.height)];
	[scrollBoardsView setContentOffset:CGPointMake(positionInLecture*scrollBoardsView.frame.size.width,0) animated:NO];
}

-(ElementButton*)setupElementButton:(NSString*)name atPosition:(int)pos
{
	ElementButton *btn = [[ElementButton alloc] init];
	btn.frame = CGRectMake(10, 10+pos*70, 80, 50);
	btn.bounds = CGRectMake(0, 0, 80, 50);
	//[btn setTitle:name forState:UIControlStateNormal];
	//[btn setTitleColor:[UIColor blackColor] forState:UIControlStateNormal];
	
	NSString *imName;
	if ([name isEqualToString:@"Proof"])
	{
		imName = @"proof_unselected.png";
	}
	else if ([name isEqualToString:@"Theorem"])
	{
		imName = @"theorem_unselected.png";
	}
	else if ([name isEqualToString:@"Definition"])
	{
		imName = @"def_unselected.png";
	}
	else if ([name isEqualToString:@"Graph"])
	{
		imName = @"graph_unselected.png";
	}
	else if ([name isEqualToString:@"Diagram"])
	{
		imName = @"diagram_unselected.png";
	}
	else if ([name isEqualToString:@"Unordered List"])
	{
		imName = @"uolist_unselected.png";
	}
	else if ([name isEqualToString:@"Ordered List"])
	{
		imName = @"olist_unselected.png";
	}
	else if ([name isEqualToString:@"Example"])
	{
		imName = @"example_unselected.png";
	}
	else if ([name isEqualToString:@"Questions and Answers"])
	{
		imName = @"qa_unselected.png";
	}
	else if ([name isEqualToString:@"Custom"])
	{
		imName = @"custom_unselected.png";
	}	
	[btn setBackgroundImage:[UIImage imageNamed:imName]];
	[btn setBackgroundColor:[scrollElementsView backgroundColor]];
	//[btn addTarget:self action:@selector(elementSelected:) forControlEvents:UIControlEventTouchUpInside];
	btn.delegate = self;
	//[btn release];
	
	return btn;
}


// Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations
	if (interfaceOrientation == UIInterfaceOrientationLandscapeLeft || interfaceOrientation == UIInterfaceOrientationLandscapeRight){
		return YES;
	}else
	{
		//Doing it this way because we may want some screens to support portrait later, like the lecture chooser.
		return NO;
	}
}

- (void)didReceiveMemoryWarning {
	// Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
	
	// Release any cached data, images, etc that aren't in use.
}


- (void)dealloc {
	if (lecture)
		[lecture release];
	if (revertImage)
		[revertImage release];
	[proofButton release];
	[theoremButton release];
	[definitionButton release];
	[graphButton release];
	[diagramButton release];
	[uoListButton release];
	[oListButton release];
	[exampleButton release];
	[qaButton release];
	[customButton release];
	[myToolbar release];
	[accept release];
	[changeLayout release];
	if (timing)
		[timing release];
	[boardPos release];
	[scrollElementsView release];
	[scrollBoardsView release];
	[boardViews release];
    [super dealloc];
}

// Daniel: This method should now return the board view for
// the CURRENT board
-(EditorBoardView*)getBoardView
{
	return [boardViews objectAtIndex:positionInLecture];
	
	/// OLD CRAP
//	for (id obj in self.view.subviews)
//	{
//		if ([obj isKindOfClass:[EditorBoardView class]])
//		{
//			return obj;
//		}
//	}
	
}

-(UIViewController*)getLectureOverviewController
{
	for (id obj in self.navigationController.viewControllers)
	{
		if ([obj isKindOfClass:[LectureOverviewViewController class]])
			return obj;
	}
	return nil;
}

-(void)popToLectureOverview
{
	UIViewController *loVC = [self getLectureOverviewController];
	loVC.navigationController.navigationBarHidden = NO;
	[self.navigationController popToViewController:loVC animated:NO];
}

#pragma mark callbacks

- (void)scrollViewDidEndDecelerating:(UIScrollView *)scrollView
{
	if (scrollView == scrollBoardsView)
	{
		positionInLecture = scrollBoardsView.contentOffset.x / scrollBoardsView.frame.size.width;
		
		// re-enable everything
		changeLayout.enabled = YES;
		accept.enabled = YES;
		add.enabled = YES;
		if (lecture.timing)
			timing.enabled = YES;
		if (lecture.physBoardPos.count > 0)
			boardPos.enabled = YES;
	}
}

- (void)scrollViewWillBeginDragging:(UIScrollView *)scrollView
{
	if (scrollView == scrollBoardsView)
	{
		// Disable everything temporarily
		changeLayout.enabled = NO;
		accept.enabled = NO;
		add.enabled = NO;
		timing.enabled = NO;
		boardPos.enabled = NO;
	}
}

-(void)deselectElement
{
	hasSelectedButton = NO;
	[selectedButton setBackgroundImage:revertImage];
	selectedButton = nil;
	[revertImage release];
	revertImage = nil;
	[[self getBoardView] setSelectedElement:NO withType:Empty];
}

-(IBAction)toolbarButtonPressed:(id)sender
{
	if (sender == changeLayout)
	{
		ChooseLayoutViewController *clVC = [[ChooseLayoutViewController alloc] initWithNibName:@"ChooseLayoutViewController" bundle:[NSBundle mainBundle]];
		clVC.model = lecture;
		clVC.positionInLecture = positionInLecture;
		clVC.existingBoard = [lecture getBoard:positionInLecture];
		clVC.makeNewBoardEditor = NO;
		[self.navigationController pushViewController:clVC animated:YES];
		[clVC release];
	}
	else if (sender == accept)
	{
		[self popToLectureOverview];
	}
	else if (sender == add)
	{
		ChooseLayoutViewController *clVC = [[ChooseLayoutViewController alloc] initWithNibName:@"ChooseLayoutViewController" bundle:[NSBundle mainBundle]];
		clVC.model = lecture;
		clVC.positionInLecture = positionInLecture+1;
		clVC.makeNewBoardEditor = NO;
		clVC.boardEditor = self;
		[self.navigationController pushViewController:clVC animated:YES];
		[clVC release];
	}
	else if (sender == timing)
	{
		TimeSetViewController *tsVC = [[TimeSetViewController alloc] initWithNibName:@"TimeSetViewController" bundle:[NSBundle mainBundle]];
		tsVC.lecture = lecture;
		tsVC.board = [lecture getBoard:positionInLecture];
		tsVC.posInLecture = positionInLecture;
		[self presentModalViewController:tsVC animated:YES];
		[tsVC release];
	}
	else if (sender == boardPos)
	{
		PhysBoardLocViewController *pblVC = [[PhysBoardLocViewController alloc] initWithNibName:@"PhysBoardLocViewController" bundle:[NSBundle mainBundle]];
		pblVC.lecture = lecture;
		pblVC.board = [lecture getBoard:positionInLecture];
		pblVC.isEditing = YES;
		[self presentModalViewController:pblVC animated:YES];
		[pblVC release];
	}
}

-(void)elementSelected:(id)sender
{
	
	// CASE: Tap the same element that was selected (to deselect)
	if (hasSelectedButton && sender == selectedButton)
	{
		[self deselectElement];
	}
	// CASE: Tap a new element to select it
	else
	{
		if (hasSelectedButton)
		{
			[selectedButton setBackgroundImage:revertImage];
			[revertImage release];
		}
		selectedButton = sender;
		hasSelectedButton = YES;
		revertImage = [selectedButton currentBackgroundImage];
		[revertImage retain];
		
		// Change the image to something else to indicate selection
		NSString *imName;
		if (selectedButton == proofButton)
		{
			imName = @"proof_selected.png";
		}
		if (selectedButton == theoremButton)
		{
			imName = @"theorem_selected.png";
		}
		if (selectedButton == definitionButton)
		{
			imName = @"def_selected.png";
		}
		if (selectedButton == graphButton)
		{
			imName = @"graph_selected.png";
		}
		if (selectedButton == diagramButton)
		{
			imName = @"diagram_selected.png";
		}
		if (selectedButton == uoListButton)
		{
			imName = @"uolist_selected.png";
		}
		if (selectedButton == oListButton)
		{
			imName = @"olist_selected.png";
		}
		if (selectedButton == exampleButton)
		{
			imName = @"example_selected.png";
		}
		if (selectedButton == qaButton)
		{
			imName = @"qa_selected.png";
		}
		if (selectedButton == customButton)
		{
			imName = @"custom_selected.png";
		}
		[selectedButton setBackgroundImage:[UIImage imageNamed:imName]];
	}
	
	// Notify the board view that there is a selected element
	ElementType sel;
	if (sender == proofButton)
		sel = Proof;
	if (sender == theoremButton)
		sel = Theorem;
	if (sender == definitionButton)
		sel = Definition;
	if (sender == graphButton)
		sel = Graph;
	if (sender == diagramButton)
		sel = Diagram;
	if (sender == uoListButton)
		sel = UOList;
	if (sender == oListButton)
		sel = OList;
	if (sender == exampleButton)
		sel = Example;
	if (sender == qaButton)
		sel = QandA;
	if (sender == customButton)
		sel = Custom;
	[[self getBoardView] setSelectedElement:hasSelectedButton withType:sel];
}


-(void) setAnnotation:(BoardSection*) bsect
{
	annotatee = bsect;
	rootPopUp = [[UIActionSheet alloc]
		      initWithTitle:@"Annotate Section"
		      delegate:self
		      cancelButtonTitle:@"Cancel"
		      destructiveButtonTitle:nil
		      otherButtonTitles:
			@"Text...",
		        @"Photo...",
		        @"Remove element", nil];
	[rootPopUp showInView:self.view];
}

-(void)actionSheet:(UIActionSheet *)actionSheet clickedButtonAtIndex:(NSInteger)buttonIndex
{
  if (actionSheet == rootPopUp) {
    switch (buttonIndex) {
    case 0: // Text
      [self setTextAnnotation];
      break;
    case 1: // Photos
	if ([annotatee hasPhoto]) {
	  photoPopUp = [[UIActionSheet alloc]
			 initWithTitle:@"Modify Photo"
			 delegate:self
			 cancelButtonTitle:@"Cancel"
			 destructiveButtonTitle:nil
			 otherButtonTitles:
			   @"Take new photo",
			   @"Load existing photo",
			   @"Remove photo", nil];
	} else {
	  photoPopUp = [[UIActionSheet alloc]
			 initWithTitle:@"Modify Photo"
			 delegate:self
			 cancelButtonTitle:@"Cancel"
			 destructiveButtonTitle:nil
			 otherButtonTitles:
			   @"Take new photo",
			   @"Load existing photo", nil];
	}
	[photoPopUp showInView:self.view];
	break;
    case 2: // Rm element
      [self removeElement];
      break;
    }
    //[rootPopUp release];
  }


  if (actionSheet == textPopUp) {
    if ([annotatee hasText]) {
      switch (buttonIndex) {
      case 0:
	[self setTextAnnotation];
	break;
      case 1:
	[self removeTextAnnotation];
	break;
      }
    } else {
      switch (buttonIndex) {
      case 0:
	[self setTextAnnotation];
	break;
      }
    }
    //[textPopUp release];
  }

  if (actionSheet == photoPopUp) {
    if ([annotatee hasPhoto]) {
      switch (buttonIndex) {
      case 0:
	[self setPhotoAnnotation:YES];
	break;
      case 1:
	[self setPhotoAnnotation:NO];
	break;
	  case 2:		  
	[self removePhotoAnnotation];
	break;
      }
    } else {
      switch (buttonIndex) {
      case 0:
	[self setPhotoAnnotation:YES];
	break;
	  case 1:
	[self setPhotoAnnotation:NO];
	break;
      }
    }
    //[photoPopUp release];
  }
}

-(void)removeElement
{
	[annotatee clearContents];
	[[self getBoardView] setNeedsDisplay];
}

-(void)setTextAnnotation
{
  if (annotate_alert == nil) {
    annotate_alert = [[UIAlertView alloc] initWithTitle:@"Section label" message:@"\n"
					  delegate:self cancelButtonTitle:nil
					  otherButtonTitles:@"Remove", @"Save", nil];
  }
    [annotate_alert setTransform:CGAffineTransformMakeTranslation(0.0, 85)];


  if (annotate_textfield == nil) {
	  annotate_textfield = [[UITextField alloc] initWithFrame:CGRectMake(12.0, 45.0, 260.0, 25.0)];
  }
  if ([annotatee hasText]) {
    annotate_textfield.text = [annotatee getTextAnnotation];
  } else {
    annotate_textfield.text = nil;
  }
	
    [annotate_textfield setBackgroundColor:[UIColor whiteColor]];
    [annotate_alert addSubview:annotate_textfield];
  


  annotate_textfield.delegate = self;
  annotate_textfield.keyboardType = UIKeyboardTypeAlphabet;
  annotate_textfield.keyboardAppearance = UIKeyboardAppearanceAlert;
  annotate_textfield.autocorrectionType = UITextAutocorrectionTypeNo;
  annotate_textfield.returnKeyType = UIReturnKeyDone;
	[annotate_textfield becomeFirstResponder];
	
  [annotate_alert show];
}

- (void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex
{
  if (alertView == annotate_alert) {
	if (buttonIndex == 0)
	{
	  [self removeTextAnnotation];
	} else if (buttonIndex == 1) {
	  [annotatee setTextAnnotation:[annotate_textfield text]];
	}
	[[self getBoardView] setNeedsDisplay];
  }
}


-(BOOL)textFieldShouldReturn:(UITextField *)textField
{
	[textField resignFirstResponder];
	
	if (textField == annotate_textfield)
	{
	  [annotate_alert dismissWithClickedButtonIndex:1 animated:YES];
	  [self alertView:annotate_alert clickedButtonAtIndex:1];
	  return YES;
	} else {
		return NO;
	}
	return NO;
}

// Threading needed
-(void)setPhotoAnnotation:(BOOL) newPhoto
{
	
	UIImagePickerController *imagePickerController = [[UIImagePickerController alloc] init];
	imagePickerController.delegate = self;
	imagePickerController.allowsImageEditing = NO;
	if (newPhoto){
		imagePickerController.sourceType = UIImagePickerControllerSourceTypeCamera;
	}else{
		imagePickerController.sourceType = UIImagePickerControllerSourceTypePhotoLibrary;
	}
	[self presentModalViewController:imagePickerController animated:NO];
	[imagePickerController release];
}


// Threading needed
- (void)imagePickerController:(UIImagePickerController *)picker 
		didFinishPickingImage:(UIImage *)image
		editingInfo:(NSDictionary *)editingInfo
{
  NSDictionary *args = [[NSDictionary alloc] initWithObjectsAndKeys:picker, @"picker",
						image, @"image", editingInfo, @"editingInfo", nil];
	
	[picker retain];
	[image retain];
	[editingInfo retain];
	
	[picker dismissModalViewControllerAnimated:YES];
	[annotatee setPhotoAnnotation:image];
	[[self getBoardView] setNeedsDisplay];
	

	[NSThread detachNewThreadSelector:@selector(imagePickerWorkerThread:)
	    toTarget:self withObject:args];
}


-(void)imagePickerWorkerThread:(NSDictionary*) args
{
	NSAutoreleasePool *arp = [[NSAutoreleasePool alloc] init];

	UIImagePickerController *picker = (UIImagePickerController*) [args objectForKey:@"picker"];
	UIImage *image = (UIImage*) [args objectForKey:@"image"];
	NSDictionary *editingInfo  = (NSDictionary*) [args objectForKey:@"editingInfo"];
  
	if (picker.sourceType == UIImagePickerControllerSourceTypeCamera){
		UIImageWriteToSavedPhotosAlbum(image, nil, nil, nil);
	}
	
	NSData *imagePNG = UIImagePNGRepresentation(image);
	
	NSString *homeDir = NSHomeDirectory();	// Create a filename
	NSDate *today = [NSDate date];
	NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
	[dateFormatter setDateFormat:@"yyy-MM-dd-hh:mm:ss"];
	NSString *currentFile = [dateFormatter stringFromDate:today];

	
	// Append dirname and filename
	NSString *imagePath = [NSString stringWithFormat:@"%@/Documents/%@.png", homeDir, currentFile];
	
	[imagePNG writeToFile:imagePath atomically: YES];
	[annotatee setPhotoFilePath:imagePath];
	NSLog(@"This is the image path %@\n", [annotatee getPhotoFilePath]);

	[picker release];
	[image release];
	[editingInfo release];

	[arp release];
}


-(void)removeTextAnnotation
{
  [annotatee removeTextAnnotation];
  [[self getBoardView] setNeedsDisplay];
}

-(void)removePhotoAnnotation
{
  [annotatee removePhotoAnnotation];
  [[self getBoardView] setNeedsDisplay];
}

-(void)removeFlag
{
  [annotatee removeFlag];
  [[self getBoardView] setNeedsDisplay];
}

- (void)navigationController:(UINavigationController *)navigationController 
		didShowViewController:(UIViewController *)viewController animated:(BOOL)animated
{
	
}

- (void)navigationController:(UINavigationController *)navigationController 
	  willShowViewController:(UIViewController *)viewController animated:(BOOL)animated
{
}



- (void)imagePickerControllerDidCancel:(UIImagePickerController *)picker
{
    [picker dismissModalViewControllerAnimated:YES];
}

@end
