//
//  EditorBoardView.m
//  WBA
//
//  Created by Class Account on 3/29/10.
//  Copyright 2010 __MyCompanyName__. All rights reserved.
//

#import "EditorBoardView.h"
#import "BoardViewController.h"


@implementation EditorBoardView

//@synthesize model;

- (id)initWithFrame:(CGRect)frame {
    if (self = [super initWithFrame:frame]) {
        // Initialization code
		hasSelectedElement = NO;
		selectedElement = Empty;
		drawRegion = CGRectMake(0, 0, frame.size.width, frame.size.height);
		separatorWidth = 2;
		flagSize = EDMODE_BOARD_FLAG_SIZE;
		thumbSize = EDMODE_BOARD_PHOTO_THUMB_SIZE;
    }
    return self;
}

- (id)initWithCoder:(NSCoder*)coder
{
	if (self = [super initWithCoder:coder]) {
        // Initialization code
		hasSelectedElement = NO;
		selectedElement = Empty;
		drawRegion = CGRectMake(EDMODE_BOARD_X, EDMODE_BOARD_Y, EDMODE_BOARD_WIDTH, EDMODE_BOARD_HEIGHT);
		separatorWidth = 2;
		flagSize = EDMODE_BOARD_FLAG_SIZE;
		thumbSize = EDMODE_BOARD_PHOTO_THUMB_SIZE;
    }
    return self;
}

-(void)setSelectedElement:(bool)hasSelection withType:(ElementType)elType
{
	hasSelectedElement = hasSelection;
	selectedElement = elType;
}

- (void)dealloc 
{
    [super dealloc];
}

# pragma mark touches

-(void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event
{	
	//[self setNeedsDisplay];
}

-(void)touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event
{
	//[self setNeedsDisplay];
}

-(void) interactWithSound:(BoardSection*) bs
{
  bsect = bs;
  soundPopUp = [[UIActionSheet alloc]
		 initWithTitle:@"Audio Note"
		 delegate:self
		 cancelButtonTitle:@"Cancel"
		 destructiveButtonTitle:nil
		 otherButtonTitles:
		   @"Play Note",
		 @"Remove Note", nil];
  [soundPopUp showInView:((BoardViewController*)controller).view];
}

-(void)actionSheet:(UIActionSheet *)actionSheet clickedButtonAtIndex:(NSInteger)buttonIndex
{
  if (buttonIndex == 0 && [bsect hasFlag]) {
	  
	  [self playBack];
	
  } else if (buttonIndex == 1) {
    [bsect removeFlag];
    [controller.view setNeedsDisplay];
    [self setNeedsDisplay];
  }
}

- (void) playBack {
	
	NSLog(@"Playing sound %@\n", bsect.soundFile);
	isPlaying = YES;
	seconds = 0;
    NSURL *fileURL = [[NSURL alloc] initFileURLWithPath:bsect.soundFile];
	NSError *err = nil;
	
	audioPlayer = [[AVAudioPlayer alloc] initWithContentsOfURL: fileURL error: &err];
    [fileURL release];
	[audioPlayer setDelegate: self];
	

	audioLength = (int) audioPlayer.duration + 1;
	NSString *timeMsg = [[NSString alloc] initWithFormat:@"                  0:00 / %d:%02d",
						 audioLength / 60, 
						 audioLength % 60];

	
	// Spawn playback timer update thread
	[NSThread detachNewThreadSelector:@selector(timerThread) toTarget:self withObject:nil];
	playbackAlert = [[[UIAlertView alloc] initWithTitle:@"Playing back audio note..."
												message:@"  " // Just some spacing
											   delegate: self
									  cancelButtonTitle:nil otherButtonTitles:@"Stop", nil] autorelease];
	
	playbackTimer = [[UILabel alloc] initWithFrame:CGRectMake(12.0, 45.0, 260.0, 25.0)];
	
	[playbackTimer setText:timeMsg];
//	[playbackTimer setText:@"                      0:00"];
	[playbackTimer setBackgroundColor:[UIColor clearColor]];
	[playbackTimer setTextColor:[UIColor whiteColor]];
	[playbackAlert addSubview:playbackTimer];
	[playbackAlert show];
	
	

	// [timeMsg release];
	
	if (![audioPlayer prepareToPlay]) {
		NSLog(@"Error at prepare to play\n");
	}    
	
	
	if (![audioPlayer play]) {
		NSLog(@"Failed to play sound\n");
	}
	
}

// Functions to deal with updating the alert view during audio note playback  
- (void) timerThread {
	while (isPlaying) {
		[NSThread sleepForTimeInterval:1.0];
		if (playbackAlert != nil) {
			seconds++;
			[self performSelectorOnMainThread : @ selector(updatePlaybackTimer) withObject:nil waitUntilDone:YES];
/*			
			// if timer's length exceeds audio file's length, dismiss alert view
			if (seconds >= (int) audioPlayer.duration + 1) {
				[self pressTheStopButton];
//				[self performSelectorOnMainThread : @ selector(pressTheStopButton) withObject:nil waitUntilDone:YES];
			}
 */
		}
	}
}

- (void) updatePlaybackTimer {
	NSString *newMsg = [[NSString alloc] initWithFormat:@"                  %d:%02d / %d:%02d",
						seconds / 60, 
						seconds % 60,
						audioLength / 60, 
						audioLength % 60];

//	NSString *newMsg = [[NSString alloc] initWithFormat:@"                      %d:%02d",
//						seconds / 60, 
//						seconds % 60];
	NSString *oldText = playbackTimer.text;
	playbackTimer.text = newMsg;
	[oldText release];
   if (seconds >= (int) audioPlayer.duration + 1) {
//		[self performSelectorOnMainThread : @ selector(pressTheStopButton) withObject:nil waitUntilDone:YES];
		isPlaying = NO;
		[playbackAlert dismissWithClickedButtonIndex:0 animated:YES];
	}
	
}

- (void) pressTheStopButton {
	[playbackAlert dismissWithClickedButtonIndex:0 animated:YES];
}

- (void)alertView:(UIAlertView *)alertView didDismissWithButtonIndex:(NSInteger)buttonIndex {
	if (buttonIndex == 0) {
		isPlaying = NO;
		[audioPlayer stop];
	} 
	
}

- (void)audioPlayerDecodeErrorDidOccur:(AVAudioPlayer *)player error:(NSError *)error
{
	NSLog(@"Error with audio player\n");
	NSLog([error localizedDescription]);
	NSLog([error localizedFailureReason]);
	NSLog([error localizedRecoveryOptions]);
	NSLog([error localizedRecoverySuggestion]);
}


-(void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event
{

	if ([self pointOnFlagButton:[[touches anyObject] locationInView:self]]) {
	// First, check if we need to toggle the revison flag
		BoardSection *sec = [self sectionContainingPoint:[[touches anyObject] locationInView:self]];
		if (sec)
		{
		  [self interactWithSound:sec];
		}
	} else if ([self pointOnPhotoButton:[[touches anyObject] locationInView:self]]) {
	// If the tap was actually on a photo thumbnail, then display the photo
		BoardSection *sec = [self sectionContainingPoint:[[touches anyObject] locationInView:self]];
		if (sec)
		{
			UIImage *img = [sec getPhotoAnnotation];
			NSLog(@"pressed the photo button");
			PresentationImageViewController *piVC
			  = [[PresentationImageViewController alloc] initWithNibName:@"PresentationImageViewController" bundle:nil];
			[piVC setImage:img];
			[controller presentModalViewController:piVC animated:NO];
			[piVC release];
			[self setNeedsDisplay];
		}
	} else {
	  // Otherwise, then invoke normal section edit actions
	  BoardSection *sec = [self sectionContainingPoint:[[touches anyObject] locationInView:self]];
	  if (sec)
	    {
	      [sec processEditorTap:selectedElement withViewController:controller];
	      hasSelectedElement = NO;
	      selectedElement    = Empty;
	      [(BoardViewController*)controller deselectElement];
	      
	      [self setNeedsDisplay];
	    }
	}
}

# pragma mark drawing

- (void)drawRect:(CGRect)rect {
    // Drawing code
	[super drawRect:rect];
	
	// Draw section text, photo icons
	for (id obj in model.sections)
	{
		[self drawSectionText:obj labelsOnly:NO];
		[self drawSectionPhotoIcon:obj];
		if ([obj hasFlag])
			[self drawSectionFlagIcon:obj flagCode:1];
	}
}


@end