//
//  IconBoardArea.m
//  WBA
//
//  Created by Class Account on 4/2/10.
//  Copyright 2010 __MyCompanyName__. All rights reserved.
//

#import "IconBoardArea.h"
#import "LectureOverviewViewController.h"

@interface IconBoardArea() 
//Private Methods
-(void)dispatchFirstTouchAtPoint:(CGPoint)touchPoint forEvent:(UIEvent *)event;
-(void)dispatchTouchEvent:(UIView *)theView toPosition:(CGPoint)position fromPosition:(CGPoint)prevPosition;
-(void)dispatchTouchEndEvent:(UIView *)theView toPosition:(CGPoint)position;

@end

@implementation IconBoardArea



// Handles the start of a touch
-(void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event
{
	LectureOverviewViewController *del = (LectureOverviewViewController *)buttonDelegate;
	
	NSUInteger numTaps = [[touches anyObject] tapCount];
	// Enumerate through all the touch objects.
	NSUInteger touchCount = 0;
	
	//After the first tap, the overlay should come on because subsequent taps mean you'll need it
	del.overlay.hidden = NO;
	
	if (numTaps >= 2)
	{	
		for (UITouch *touch in touches) {
		// Send to the dispatch method, which will make sure the appropriate subview is acted upon
		[self dispatchFirstTouchAtPoint:[touch locationInView:self] forEvent:event];
		touchCount++;  
		}	
	}
}


//Starts the process of dragging the selected button but bringing it out of the list of
//boards
-(void)dispatchFirstTouchAtPoint:(CGPoint)touchPoint forEvent:(UIEvent *)event
{
	LectureOverviewViewController *del = (LectureOverviewViewController *)buttonDelegate;
	
	emptyBoard = [[Board alloc] initWithLayoutType:-1];
	for (IconBoardView *btn in self.subviews){
		if (CGRectContainsPoint([btn frame], touchPoint)) {
			
			[del.scrollLectureView setScrollEnabled:NO];
			[self bringSubviewToFront:btn];
			curBoard = btn;
			int replace = [del.model.boardList indexOfObject:btn.model];
			
			[del.model.boardList replaceObjectAtIndex:replace withObject:emptyBoard];
			[del updateBoards];
			[del.boardsArea addSubview:curBoard];
		}
	}
}

// Handles the continuation of a touch.
-(void)touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event
{  
	
	NSUInteger touchCount = 0;
	// Enumerates through all touch objects
	for (UITouch *touch in touches) {
		// Send to the dispatch method, which will make sure the appropriate subview is acted upon
		[self dispatchTouchEvent:[touch view] toPosition:[touch locationInView:self] fromPosition:[touch previousLocationInView:self]];
		touchCount++;
	}
}


-(void)dispatchTouchEvent:(UIView *)theView toPosition:(CGPoint)position fromPosition:(CGPoint)prevPosition
{
	LectureOverviewViewController *del = (LectureOverviewViewController *)buttonDelegate;

	// Check to see which view, or views,  the point is in and then move to that position.
	for (IconBoardView *btn in self.subviews){
		if ([btn class] == [IconBoardView class] && btn.isBeingDragged && CGRectContainsPoint([btn frame], position)) {
			
			//horizontal motion
			int newIndex = (int)[self getBoardPosition:btn.center];
			
			if (8*fabs(position.x - prevPosition.x) > fabs(position.y - prevPosition.y)){
				btn.center = CGPointMake(position.x, btn.center.y);
			}else{
				btn.center = CGPointMake(btn.center.x, position.y);
			}
				[del.scrollLectureView scrollRectToVisible:CGRectMake(btn.center.x-((ICONMODE_BOARD_WIDTH + ICONMODE_BOARD_SPACING)/2), 0, ICONMODE_BOARD_WIDTH+ICONMODE_BOARD_SPACING, 200) animated:NO];
				if(newIndex >=0)
				{
					//remove delete mark if it's there
					if (curBoard.subviews.count > 0){
						[[curBoard.subviews objectAtIndex:0] removeFromSuperview];
					}
					if ([del.model.boardList objectAtIndex:newIndex] != emptyBoard) {
						[del.model.boardList removeObject:emptyBoard];
					
						[del.model.boardList insertObject:emptyBoard atIndex:newIndex];
						[del updateBoards];
						[del.boardsArea addSubview:curBoard];
					}//endif board isn't emptyBoard
				}// endif in an index
				else{
					
					if ([del.model numBoards] > 1){//If not edge case
					[del.model.boardList removeObject:emptyBoard];
					}
					
					//Add a delete X
					UIImageView *deleteMark = [[UIImageView alloc] initWithImage:[UIImage imageNamed:@"x.png"]];
					int deleteXPos = deleteMark.frame.size.width * -0.25;
					int deleteYPos = deleteMark.frame.size.height * -0.25;
					if (newIndex == -1) { deleteYPos = deleteYPos * -1 + (curBoard.frame.size.height - deleteMark.frame.size.height);}
					deleteMark.frame = CGRectMake(deleteXPos,deleteYPos , deleteMark.frame.size.width, deleteMark.frame.size.height);
					if (curBoard.subviews.count == 0)
						[curBoard addSubview:deleteMark];
					
					
					[del updateBoards];
					[del.boardsArea addSubview:curBoard];
					//NSLog(@"number of sv: %d", btn.subviews.count);
					
				}//endelse deleting
			
			//Break to assure you're only doing this once
			break;
		}//endif this is the board being acted on
	}//endfor all boards
}//endmethod

-(int) getBoardPosition:(CGPoint)p {
	LectureOverviewViewController *del = (LectureOverviewViewController *)buttonDelegate;

	for (int i = 0; i < [del.model numBoards]; i++){
		CGRect boardLoc = CGRectMake(i*(ICONMODE_BOARD_WIDTH + ICONMODE_BOARD_SPACING), ICONMODE_BOARD_Y, ICONMODE_BOARD_WIDTH+ICONMODE_BOARD_SPACING, ICONMODE_BOARD_HEIGHT)	;
		if (CGRectContainsPoint(boardLoc, p)){
			return i;
		}
	}
	//Board is in delete range
	if (p.y < ICONMODE_BOARD_Y){
		return -1;
	}
	else if (p.y > ICONMODE_BOARD_Y + ICONMODE_BOARD_HEIGHT){
		return -2;
	}
	return [del.model numBoards] - 1;
}

// Handles the end of a touch event.
-(void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event
{
	// Enumerates through all touch object
	for (UITouch *touch in touches) {
		// Sends to the dispatch method, which will make sure the appropriate subview is acted upon
		[self dispatchTouchEndEvent:[touch view] toPosition:[touch locationInView:self]];
	}
}


-(void)dispatchTouchEndEvent:(UIView *)theView toPosition:(CGPoint)position
{   
	LectureOverviewViewController *del = (LectureOverviewViewController *)buttonDelegate;

	for (IconBoardView *btn in self.subviews){
		if ([btn class] == [IconBoardView class] && btn.isBeingDragged && CGRectContainsPoint([btn frame], position)) {
			int newIndex = (int)[self getBoardPosition:btn.center];
			while ([del.model.boardList containsObject:emptyBoard]){
				[del.model.boardList removeObject:emptyBoard];
			}
			
			if(newIndex >=0)
			{
				[del.model.boardList removeObject:btn.model];
				[del.model addBoard:btn.model atPosition:newIndex];
			}
			[del updateBoards];
		}
	}
}



-(UIViewController *) getButtonDelegate{
	return buttonDelegate;
}

-(void) setButtonDelegate:(LectureOverviewViewController *)del {
	buttonDelegate = del;
	for (IconBoardView *btn in self.subviews){
		if ([btn class] == [IconBoardView class])
			btn.delegate = buttonDelegate;
	}
}


	
	

- (void)dealloc {
    [super dealloc];
	[emptyBoard release];
	[curBoard release];
}


@end
