//
//  IconBoardView.m
//  WBA
//
//  Created by Class Account on 4/2/10.
//  Copyright 2010 __MyCompanyName__. All rights reserved.
//

#import "IconBoardView.h"
#import "LectureOverviewViewController.h"


@implementation IconBoardView

@synthesize isBeingDragged, originDragIndex, dragIndex, boardIndex;
@synthesize delegate;
@synthesize deleteMark;




- (id)initWithFrame:(CGRect)frame {
    if (self = [super initWithFrame:frame]) {
        // Initialization code
		drawRegion = CGRectMake(0, 0, ICONMODE_BOARD_WIDTH, ICONMODE_BOARD_HEIGHT);
		separatorWidth = ICONMODE_LINEWIDTH;
    }


    return self;
}

- (id)initWithCoder:(NSCoder*)coder {
    if (self = [super initWithCoder:coder]) {
        // Initialization code
		drawRegion = CGRectMake(ICONMODE_BOARD_X, ICONMODE_BOARD_Y, ICONMODE_BOARD_WIDTH, ICONMODE_BOARD_HEIGHT);
		separatorWidth = ICONMODE_LINEWIDTH;
    }
    return self;
}

/* Handles the start of a touch
   Cancels event for single tap if it turns out to be a double tap */
-(void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event
{
	LectureOverviewViewController *del = (LectureOverviewViewController *)delegate;
	[self.superview bringSubviewToFront:self];
	
	if ([del editMode])
	{
		[self startDrag];
		[self animateStartDrag];
		return;
	}
	
	
	NSValue *point = [NSValue valueWithCGPoint:[[touches anyObject] locationInView:del.view]];
	
	touchTimer = [NSTimer scheduledTimerWithTimeInterval:0.7 
												  target:self 
												selector:@selector(setupDrag:) 
												userInfo:point
												 repeats:NO];

}


-(void)setupDrag:(NSTimer *)timer{
	LectureOverviewViewController *del = (LectureOverviewViewController *)delegate;
	//order important
	[self startDrag];
	[del startEditing];
	//endorder
	[UIView beginAnimations:nil context:NULL];
	[UIView setAnimationDuration:0.5];
	self.frame = CGRectMake([[touchTimer userInfo] CGPointValue].x - (ICONMODE_EDIT_BOARD_WIDTH * 1.3)/2, [[touchTimer userInfo] CGPointValue].y -  (ICONMODE_EDIT_BOARD_HEIGHT* 1.3)/2, ICONMODE_EDIT_BOARD_WIDTH * 1.3, ICONMODE_EDIT_BOARD_HEIGHT * 1.3);
	//CGPoint newCenter = CGPointMake([[touchTimer userInfo] CGPointValue].x, [[touchTimer userInfo] CGPointValue].y);
	//self.center = newCenter;
	for (UIView *v in self.subviews){
		v.frame = CGRectMake(self.frame.size.width - v.frame.size.width*.75, v.frame.size.height*-.25, v.frame.size.width, v.frame.size.height); 
	}
	[UIView commitAnimations];	
	
	[self invalidateTimer];
	


	
}

-(void)startDrag{
	self.isBeingDragged = YES;
	originDragIndex = dragIndex = [self getBoardPosition:self.center];
	[self.deleteMark removeFromSuperview];
	
}



// Handles the continuation of a touch.
// Actually doesn't do anything but here for posterity and debugging.
-(void)touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event
{  
	
	LectureOverviewViewController *del = (LectureOverviewViewController *)delegate;
	if ([del editMode])
	{
		self.isBeingDragged = YES;
		[[del scrollLectureView] setScrollEnabled:NO];	
	}
	if (isBeingDragged){
		self.center = [[touches anyObject] locationInView:self.superview]; 
		int newIndex = [self getBoardPosition:self.center];
		[[del scrollLectureView] scrollRectToVisible:CGRectMake(self.frame.origin.x - ICONMODE_EDIT_BOARD_WIDTH, 0, ICONMODE_EDIT_BOARD_WIDTH*3, ICONMODE_EDIT_BOARD_HEIGHT) animated:NO];
		
		
		[self invalidateTimer];
		if (CGRectContainsRect([del deleteArea].frame, self.frame)){
				//nothing
		}
		else {
			if (![del.model.boardList containsObject:self.model]){
				[del.model.boardList addObject:self.model];
			}
		}
		
		if(newIndex >=0 && newIndex != dragIndex)
		{
			dragIndex = newIndex;
			[[del.model boardList] removeObject:self.model];
			[[del.model boardList] insertObject:self.model atIndex:newIndex];
			[UIView beginAnimations:nil context:nil];
			[UIView setAnimationDuration:0.09];
			[del refreshBoards];
			[UIView commitAnimations];
			

		}
	}
}

// Handles the end of a touch event.
-(void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event
{
	LectureOverviewViewController *del = (LectureOverviewViewController *)delegate;
	[self animateEndDrag];
	[self invalidateTimer];
	if (!isBeingDragged) {
		[self selectBoard];
	}
	if (CGRectContainsRect([del deleteArea].frame, self.frame)){
		[del.model.boardList removeObject:self.model];
		self.frame = CGRectMake(self.frame.origin.x, self.frame.origin.y, ICONMODE_EDIT_BOARD_WIDTH, ICONMODE_EDIT_BOARD_HEIGHT);
		for (UIView *v in self.subviews){
			v.frame = CGRectMake(self.frame.size.width - v.frame.size.width*0.75, v.frame.size.height*-0.25, v.frame.size.width, v.frame.size.height); 
		}
		[self shrinkSmaller];
		deleteMark = [[UIImageView alloc] initWithImage:[UIImage imageNamed:@"x.png"]];
		deleteMark.frame = CGRectMake(-10, -10, deleteMark.frame.size.width / 2, deleteMark.frame.size.height / 2);
		[self addSubview:deleteMark];
		//[self removeFromSuperview];
	}
	
	isBeingDragged = NO;
	[[del scrollLectureView] setScrollEnabled:YES];
	[UIView beginAnimations:nil context:nil];
	[UIView setAnimationDuration:0.09];
	[del refreshBoards];	
	[UIView commitAnimations];
}



-(void)animateStartDrag
{
	// Pulse the view by scaling up, then move the view to under the finger.
	[UIView beginAnimations:nil context:nil];
	[UIView setAnimationDuration:0.1];
	self.transform = CGAffineTransformMakeScale(1.3, 1.3);
	for (UIView *v in self.subviews){
		v.transform = CGAffineTransformMakeScale(1.3, 1.3);
	}
	[UIView commitAnimations];
}

// Scales down the view
-(void)animateEndDrag
{	
	[UIView beginAnimations:nil context:NULL];
	[UIView setAnimationDuration:0.1];
	// Set the transform back to the identity, thus undoing the previous scaling effect.
	self.transform = CGAffineTransformIdentity;
	for (UIView *v in self.subviews){
		v.transform = CGAffineTransformIdentity;
	}
	[UIView commitAnimations];	
}

-(void)shrinkSmaller{
	[UIView beginAnimations:nil context:nil];
	[UIView setAnimationDuration:0.1];
	self.transform = CGAffineTransformMakeScale(0.75, 0.75);
	for (UIView *v in self.subviews){
		v.transform = CGAffineTransformMakeScale(0.75,0.75);
	}
	[UIView commitAnimations];
}	


-(void)selectBoard{
	
	LectureOverviewViewController *del = (LectureOverviewViewController *)delegate;
	
	//Find position of this board
	int pos = [self getBoardPosition:self.center];
	
	[del boardSelected:self.model atPosition:pos];
}

-(int) getBoardPosition:(CGPoint)p {
	
	LectureOverviewViewController *del = (LectureOverviewViewController *)delegate;

	int width = [del editMode] ? ICONMODE_EDIT_BOARD_WIDTH : ICONMODE_BOARD_WIDTH;
	int spacing = [del editMode] ? ICONMODE_EDIT_BOARD_SPACING : ICONMODE_BOARD_SPACING;	
	
	
	for (int i = 0; i < [del.model numBoards]; i++){
		CGRect boardLoc = CGRectMake(i*(width + spacing), 0, width+spacing, [del scrollLectureView].frame.size.height);
		if (CGRectContainsPoint(boardLoc, p)){
			return i;
		}
	}
	return -1;
}

-(void)invalidateTimer{
	if (touchTimer != nil){
		[touchTimer invalidate];
		touchTimer = nil;
		NSLog(@"Invalidated!");
	}
}


- (void)drawRect:(CGRect)rect {
    // Drawing code
	
	[super drawRect:rect];
	
	// Draw ONE flag icon if the board has any flagged elements
	for (id obj in model.sections)
	{
		[self drawFlagForWholeBoard];
	}
	
}

-(void)drawFlagForWholeBoard
{
	NSString *soundFile = nil;
	for (id obj in model.sections)
	{
		if ([obj hasFlag])
		{
		  soundFile = ((BoardSection*)obj).soundFile;
			
			break;
		}
	}
	
	if (soundFile != nil)
	{
		// Make a dummy section that has a flag, invoke drawSectionFlagIcon
		BoardSection *dummy = [[BoardSection alloc] init];
		dummy.region = CGRectMake(0, 0, 1, 1);
		dummy.type = Proof;
		dummy.soundFile = soundFile;
		[dummy.soundFile retain];
		[self drawSectionFlagIcon:dummy flagCode:0];
		[dummy release];
	}
}



- (void)dealloc {
    [super dealloc];
	[delegate dealloc];
	[deleteMark release];
}


@end
