//
//  Lecture.m
//  WBA
//
//  Created by Class Account on 3/29/10.
//  Copyright 2010 __MyCompanyName__. All rights reserved.
//

#import "Lecture.h"


@implementation Lecture

@synthesize lid,title,boardList,duration,timing,physBoardPos;

-(id)init
{
	if ([super init])
	{
		boardList = [[NSMutableArray alloc] init];
		physBoardPos = [[NSMutableArray alloc] init];
		timing = NO;
		duration = 50;
		title = [NSString stringWithString:EMPTY_LECTURE_TITLE];
		return self;

	}
	return nil;
}

-(id)initWithDict:(NSDictionary*) dict
{
  [super init];
  lid = [[NSString stringWithString:[dict objectForKey:LEC_ID_KEY]] retain];
  title = [NSString stringWithString:[dict objectForKey:LEC_TITLE_KEY]];
  timing = (BOOL) [[dict objectForKey:LEC_TIMING_KEY] intValue];
  duration = [[dict objectForKey:LEC_DURATION_KEY] intValue];
  physBoardPos = [[NSMutableArray alloc] init];

  for (NSDictionary *d in [dict objectForKey: LEC_BOARDS_POS_KEY]) {
    [d retain];
    CGPoint origin = CGPointMake([((NSString*)[d objectForKey: LEC_PHYSBOARDPOS_X])
				   floatValue], 
				 [((NSString*) [d objectForKey: LEC_PHYSBOARDPOS_Y]) 
				   floatValue]);
    [physBoardPos addObject:[NSValue valueWithCGPoint:origin]];
    [d release];
  }
  
  boardList = [[NSMutableArray alloc] init];
  for (NSDictionary *d in [dict objectForKey: LEC_BOARDS_KEY]) {
    [d retain];
    [boardList addObject: [[[Board alloc] initWithDict:d] retain]];
    [d release];
  }
  return self;
}


-(void) setLecId {
  if (! lid) {
    NSDate *today = [NSDate date];
    NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
    [dateFormatter setDateFormat:@"yyy-MM-dd-hh:mm:ss"];
    lid = [dateFormatter stringFromDate:today];
    [lid retain];
    [dateFormatter release];
  }
}


-(NSMutableDictionary*)serialize {
  NSMutableDictionary *lectureDictionary = [[NSMutableDictionary alloc] init];
  if (! lid ) {
    [self setLecId];
  }
  [lectureDictionary setObject: lid forKey: LEC_ID_KEY];

  [lectureDictionary setObject: (title ? title : @"") forKey: LEC_TITLE_KEY];
  [lectureDictionary setObject:[[NSString alloc] initWithFormat:@"%d", timing] 
		     forKey: LEC_TIMING_KEY];
  [lectureDictionary setObject:[[NSString alloc] initWithFormat:@"%d", duration]
		     forKey: LEC_DURATION_KEY];


  NSMutableArray *physBoardPosArray = [[NSMutableArray alloc] init];
  for (NSValue *v in physBoardPos)
    {
      CGPoint origin = [v CGPointValue];
      NSMutableDictionary *d = [[NSMutableDictionary alloc] initWithObjectsAndKeys:
								[[NSString alloc] initWithFormat:@"%f", origin.x],
                                 LEC_PHYSBOARDPOS_X,
								[[NSString alloc] initWithFormat:@"%f", origin.y], 
				 LEC_PHYSBOARDPOS_Y, 
				 nil];
      [physBoardPosArray addObject:d];
    }
  [lectureDictionary setObject:physBoardPosArray forKey: LEC_BOARDS_POS_KEY];
	

  NSMutableArray *boardArray = [[NSMutableArray alloc] init];
  for (Board * board in boardList) {
    [boardArray addObject:[board serialize]];
  }
  [lectureDictionary setObject:boardArray forKey: LEC_BOARDS_KEY];

  [lectureDictionary retain];
  return lectureDictionary;

}


-(int)numBoards
{
	return boardList.count;
}
-(void)addBoard:(Board*)board atPosition:(int)pos
{
	[boardList insertObject:board atIndex:pos];
}

-(Board*)getBoard:(int)boardNum
{
	return [boardList objectAtIndex:boardNum];
}


-(void)dealloc
{
	if (title)
		[title release];
	[boardList release];
	[physBoardPos release];
	[super dealloc];
}

@end
