//
//  LectureOverviewViewController.m
//  WBA
//
//  Created by Class Account on 3/18/10.
//  Copyright 2010 __MyCompanyName__. All rights reserved.
//

#import "LectureOverviewViewController.h"
#import "ChooseLayoutViewController.h"
#import "HelpViewController.h"
#import "PresentationViewController.h"
#import "LectureSettingsViewController.h"



@implementation LectureOverviewViewController

@synthesize scrollLectureView, deleteArea;
@synthesize startPresentationButton, bottomBar;
@synthesize model;
@synthesize editMode;

UIBarButtonItem *doneEditingButton;
UIButton *addbutton;
UIImageView *trash;
NSArray *normalBottomBarItems;
NSArray *editModeBottomBarItems;
UILabel *editInstr;



/*
 // The designated initializer.  Override if you create the controller programmatically and want to perform customization that is not appropriate for viewDidLoad.
- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
    if (self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil]) {
        // Custom initialization
    }
    return self;
}
*/


// UI stuff that needs to be recomputed every time this view shows up
-(void)viewWillAppear:(BOOL)animated
{
	[super viewWillAppear:animated];
	
	self.navigationItem.title = model.title;
	self.navigationController.navigationBarHidden = NO;
	editMode = FALSE;
	[scrollLectureView setScrollEnabled:YES];

	[self updateBoards];
}

-(void)viewDidAppear:(BOOL)animated
{
	[super viewDidAppear:animated];
	[scrollLectureView flashScrollIndicators];
}

// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad {
    [super viewDidLoad];
	
	addbutton = [[UIButton buttonWithType:UIButtonTypeRoundedRect] retain];
	[addbutton setImage:[UIImage imageNamed:@"plustext.png"] forState:UIControlStateNormal];
	[addbutton addTarget:self action:@selector(addBoard) forControlEvents:UIControlEventTouchUpInside];
	
	doneEditingButton = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemDone target:self action:@selector(doneEditing)];
	normalBottomBarItems =  [[NSArray alloc] initWithArray:bottomBar.items];
	editModeBottomBarItems = [[NSArray alloc] initWithArray:[bottomBar.items arrayByAddingObject:doneEditingButton]];
	
	deleteArea = [[UIView alloc] init];
	self.navigationItem.backBarButtonItem.title = @"Back";	
	//Set up persistent elements
	settingsButton = [[UIBarButtonItem alloc] initWithTitle:@"Settings" style:UIBarButtonItemStylePlain target:self action:@selector(showSettings)];
	self.navigationItem.rightBarButtonItem = settingsButton;
	
	//boardsArea = [[IconBoardArea alloc] init];
	//[scrollLectureView addSubview:boardsArea];

	
	editInstr = [[UILabel alloc] initWithFrame:CGRectMake(scrollLectureView.frame.size.width/2-130, ICONMODE_BOARD_Y + ICONMODE_BOARD_HEIGHT, 300, 50)];
	[editInstr setText:@"Hold Board to Rearrange or Delete"];
	
	[editInstr setTextColor:[UIColor grayColor]];
	[editInstr setBackgroundColor:[UIColor clearColor]];
	
	[self updateBoards];
	

	// Set up lecture-specific elements
	[self viewWillAppear:NO];
}
	
-(void) updateBoards {
	//Remove boards
	while ([scrollLectureView.subviews count] > 0){
		[[scrollLectureView.subviews lastObject] removeFromSuperview];	
	}
	
	//Add them all back
	for (int i = 0; i < [model numBoards]; i++)
	{
		IconBoardView *b = [[IconBoardView alloc] init];
		b.model = [model getBoard:i];
		b.delegate = self;
		UILabel *num = [[UILabel alloc] initWithFrame:CGRectMake(ICONMODE_BOARD_WIDTH, 0, 28, 30)];
		UIImageView *numBadge = [[UIImageView alloc] initWithImage:[UIImage imageNamed:@"badge.png"]];
		numBadge.frame = CGRectMake(ICONMODE_BOARD_WIDTH, 0, 30, 30);
		num.text = [NSString stringWithFormat:@"%d", i+1];
		[num setBackgroundColor:[UIColor clearColor]];
		[num setAdjustsFontSizeToFitWidth:YES];
		[num setTextColor:[UIColor blackColor]];
		//[num setShadowColor:[UIColor whiteColor]];
		//[num setShadowOffset:CGSizeMake(1,1)];
		[num setTextAlignment:UITextAlignmentCenter];
		[num setFont:[UIFont fontWithName:@"Arial" size:27]];
		
		[b addSubview:numBadge];
		[b addSubview:num];
		[self arrangeBoard:b atIndex:i];
		[scrollLectureView addSubview:b];
	}
	
	//Ensure correctness
	[self refreshBoards];
	
	//Put the Add Slide button
	addbutton.frame = CGRectMake([model numBoards]*(ICONMODE_BOARD_WIDTH + ICONMODE_BOARD_SPACING) + 200, ICONMODE_BOARD_Y, ICONMODE_BOARD_WIDTH, ICONMODE_BOARD_HEIGHT);
	[scrollLectureView addSubview:addbutton];
	[UIView beginAnimations:nil context:NULL];
	[UIView setAnimationDuration:0.2];
	addbutton.frame = CGRectMake([model numBoards]*(ICONMODE_BOARD_WIDTH + ICONMODE_BOARD_SPACING), ICONMODE_BOARD_Y, ICONMODE_BOARD_WIDTH, ICONMODE_BOARD_HEIGHT);
	[UIView commitAnimations];
	[scrollLectureView addSubview:editInstr];

	
	if ([model numBoards] == 0){
		startPresentationButton.enabled = NO;
	}
	else {
		startPresentationButton.enabled = YES;
	}
	
}


/** 
	Finds the IconBoardView in the scrollview that has this board as its model.
	If not found, returns nil.
 */
-(IconBoardView *)getIconBoardViewForBoard:(Board *) b{
	for (UIView* view in scrollLectureView.subviews){
		if ([view class] == [IconBoardView class]){
			IconBoardView *ibv = (IconBoardView *)view;
			if (ibv.model == b){
				return ibv;
			}
		}
	}
	
	return nil;
}

-(void) refreshBoards {
	for (int i = 0; i < [model numBoards]; i++){
		IconBoardView *b = [self getIconBoardViewForBoard:[model getBoard:i]];
		if (!(b.isBeingDragged))
			[self arrangeBoard:b atIndex:i];
	}
	
	[self setScrollContentSize];
}

-(void) setScrollContentSize
{
	int width = editMode ? ICONMODE_EDIT_BOARD_WIDTH : ICONMODE_BOARD_WIDTH;
	int spacing = editMode ? ICONMODE_EDIT_BOARD_SPACING : ICONMODE_BOARD_SPACING;
	int scrollContentWidth = ([model numBoards] + 1) * (width + spacing);
	if (scrollLectureView.frame.size.width > scrollContentWidth){
		scrollContentWidth = scrollLectureView.frame.size.width + 1;
	}
	[scrollLectureView setContentSize: CGSizeMake(scrollContentWidth, scrollLectureView.frame.size.height - 1)];
}

-(void)arrangeBoard:(IconBoardView*)board atIndex:(int)index{
	int width = editMode ? ICONMODE_EDIT_BOARD_WIDTH : ICONMODE_BOARD_WIDTH;
	int height = editMode ? ICONMODE_EDIT_BOARD_HEIGHT : ICONMODE_BOARD_HEIGHT;
	int spacing = editMode ? ICONMODE_EDIT_BOARD_SPACING : ICONMODE_BOARD_SPACING;
	//int vertical = editMode ? ICONMODE_BOARD_Y + ICONMODE_BOARD_HEIGHT/2 - ICONMODE_EDIT_BOARD_HEIGHT/2 : ICONMODE_BOARD_Y;
	
	
	board.frame = CGRectMake(index*(width + spacing), ICONMODE_BOARD_Y, width, height);
	board.bounds = CGRectMake(0, 0, width, height);
	board.transform = CGAffineTransformIdentity;
	for (UIView *v in board.subviews){
		v.frame = CGRectMake(width-3*v.frame.size.width/4, -1 * v.frame.size.height/4, v.frame.size.width, v.frame.size.height);
		v.bounds = CGRectMake(0, 0, v.frame.size.width, v.frame.size.height);
		v.transform = CGAffineTransformIdentity;
	}
	
	
}

-(void)startEditing{ 
	editMode = TRUE;
	
	UIBarButtonItem *empty = [[UIBarButtonItem alloc] initWithCustomView:[[UIView alloc] init]];
	[self navigationItem].leftBarButtonItem = empty;
	[empty release];
	bottomBar.items = editModeBottomBarItems;
	settingsButton.enabled = NO;
	startPresentationButton.enabled = NO;
	[self setScrollContentSize];
	
	[UIView beginAnimations:nil context:NULL];
	[UIView setAnimationDuration:0.5];
	[self refreshBoards];
	addbutton.center = CGPointMake(scrollLectureView.contentSize.width + 1000, addbutton.center.y);
	[UIView commitAnimations];
	[scrollLectureView setContentOffset:CGPointMake(0,0) animated:YES];
	//[self performSelector:@selector(refreshBoards) withObject:nil afterDelay:0.5];
	NSLog(@"animating in");
	
	
	//Set up delete area
	deleteArea.frame = CGRectMake(-1000, 
								  scrollLectureView.frame.size.height, 
								  scrollLectureView.contentSize.width + 2000, 
								  scrollLectureView.frame.size.height);
	deleteArea.backgroundColor = [UIColor lightGrayColor];
	if (deleteArea.subviews.count < 1){
		trash = [[UIImageView alloc] initWithImage:[UIImage imageNamed:@"trash.png"]];
		trash.frame = CGRectMake(1000 + scrollLectureView.frame.size.width/2 - trash.frame.size.width/2, 30, trash.frame.size.width, trash.frame.size.height);
		[deleteArea addSubview:trash];
		[trash release];
	}
	[scrollLectureView addSubview:deleteArea];
	[scrollLectureView sendSubviewToBack:deleteArea];
	
	
	[UIView beginAnimations:nil context:NULL];
	[UIView setAnimationDuration:0.5];
	deleteArea.transform = CGAffineTransformMakeTranslation(0, -1 * (50 +ICONMODE_BOARD_HEIGHT/2));
	[UIView commitAnimations];
	[scrollLectureView sendSubviewToBack:editInstr];
	NSLog(@"end of start");


}

-(void)doneEditing{
	editMode = FALSE;
	
	[self navigationItem].leftBarButtonItem = nil;
	settingsButton.enabled = YES;
	bottomBar.items = normalBottomBarItems;
	[scrollLectureView setScrollEnabled:YES];
	
	
	[UIView beginAnimations:nil context:NULL];
	[UIView setAnimationDuration:0.5];

	[self refreshBoards];
	
	[UIView commitAnimations];
	[self performSelector:@selector(updateBoards) withObject:nil afterDelay:0.5];
	NSLog(@"animating out");
	
	//Hide delete area and all views in it
	[UIView beginAnimations:nil context:NULL];
	[UIView setAnimationDuration:0.5];
	for (UIView* v in scrollLectureView.subviews){
		if ([v class] == [IconBoardView class] && CGRectContainsRect(deleteArea.frame, v.frame)){
			v.transform = CGAffineTransformMakeTranslation(0, (50 +ICONMODE_BOARD_HEIGHT/2));
		}
	}
	deleteArea.transform = CGAffineTransformIdentity;
	[UIView commitAnimations];

	[deleteArea performSelector:@selector(removeFromSuperview) withObject:nil afterDelay:0.5];	
}

int numScrolls = 0;
int oldContentOffset = 0;
- (void)scrollViewDidScroll:(UIScrollView *)scrollView{
	if (editMode){
		trash.frame = CGRectMake(1000 + scrollView.contentOffset.x + scrollLectureView.frame.size.width/2 - trash.frame.size.width/2, 30, trash.frame.size.width, trash.frame.size.height);
		for (UIView* v in scrollView.subviews){
			if ([v class] == [IconBoardView class] && CGRectContainsRect(deleteArea.frame, v.frame)){
				v.center = CGPointMake(v.center.x - oldContentOffset + scrollView.contentOffset.x, v.center.y);
			}
		}
	}
	if (++numScrolls > 2){
		for (int i = 0; i < [model numBoards]; i++){
			IconBoardView *ibv = [self getIconBoardViewForBoard:[model getBoard:i]];
			[ibv invalidateTimer];
		}
		numScrolls = 0;
	}

	oldContentOffset = scrollView.contentOffset.x;
	editInstr.frame = CGRectMake(scrollView.frame.size.width/2-130 + scrollView.contentOffset.x, ICONMODE_BOARD_Y + ICONMODE_BOARD_HEIGHT, 300, 50);
	

}

-(void)simulateEditorDonePlusBoardSelected:(BoardViewController*)bVC
{
	NSLog(@"simulating done + reselect");
	[bVC toolbarButtonPressed:bVC.accept];
	[self boardSelected:[model getBoard:bVC.positionInLecture] atPosition:bVC.positionInLecture];
}

-(void)boardSelected:(Board *)board atPosition:(int)pos{
	BoardViewController *bVC = [[BoardViewController alloc] initWithNibName:@"BoardViewController" bundle:nil];
	[self.navigationController pushViewController:bVC animated:NO];
	bVC.lecture = model;
	bVC.positionInLecture = pos;
	[bVC release];
	
}

-(void)settingsAccepted
{
	[self dismissModalViewControllerAnimated:YES];
	[self.modalViewController release];
}

-(void)showSettings
{
	LectureSettingsViewController *lsVC = [[LectureSettingsViewController alloc] initWithNibName:@"LectureSettingsViewController" bundle:[NSBundle mainBundle]];
	lsVC.parent = self;
	[lsVC setModel:model];
	[self presentModalViewController:lsVC animated:YES];
}

-(IBAction)showHelp
{
	HelpViewController *hVC = [[HelpViewController alloc] initWithNibName:@"HelpViewController" bundle:[NSBundle mainBundle]];
	hVC.parentController = self;
	[self presentModalViewController:hVC animated:YES];
	[hVC release];
}

-(IBAction) addBoard{
	// Add a new board to the END of the lecture
	ChooseLayoutViewController *clVC = [[ChooseLayoutViewController alloc] initWithNibName:@"ChooseLayoutViewController" bundle:nil];
	clVC.model = model;
	clVC.positionInLecture = [model numBoards];
	clVC.makeNewBoardEditor = YES;
	[self.navigationController pushViewController:clVC animated:YES];
	[clVC release];
}


-(IBAction) startPresentation
{
	PresentationViewController *pVC = [[PresentationViewController alloc] initWithNibName:@"PresentationViewController" bundle:[NSBundle mainBundle]];
	[pVC setLecture:model];
	[self.navigationController pushViewController:pVC animated:NO];
	[pVC release];
}

// Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations
	if (interfaceOrientation == UIInterfaceOrientationLandscapeLeft || interfaceOrientation == UIInterfaceOrientationLandscapeRight){
		return YES;
	}else
	{
		//Doing it this way because we may want some screens to support portrait later, like the lecture chooser.
		return NO;
	}
}

- (void)didReceiveMemoryWarning {
	// Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
	
	// Release any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload {
	// Release any retained subviews of the main view.
	// e.g. self.myOutlet = nil;
}


- (void)dealloc {
	[settingsButton release];
	[addbutton release];
	[normalBottomBarItems release];
	[editModeBottomBarItems release];
	[editInstr release];
	[scrollLectureView release];
	[model release];
	[startPresentationButton release];
	[model release];
	[deleteArea release];
	[doneEditingButton release];
    [super dealloc];
}


@end
