//
//  LectureSerializer.m
//  WBA
//
//  Created by Class Account on 4/30/10.
//  Copyright 2010 __MyCompanyName__. All rights reserved.
//

#import "LectureSerializer.h"

@implementation LectureSerializer

BoardSection * loadBoardSection(NSMutableDictionary * boardSectionDict, BoardSection * loadedBoardSection) {
	NSString * text = [boardSectionDict objectForKey:@"text"];
	NSString * photo = [boardSectionDict objectForKey:@"photoAnnotation"];
	NSString * audio = [boardSectionDict objectForKey:@"audioAnnotation"];
	
	[loadedBoardSection setType:(ElementType)[[boardSectionDict objectForKey:@"elementType"] intValue]];
	
	if (text != nil) {
		[loadedBoardSection setTextAnnotation:text];
		[loadedBoardSection setHasText:YES];
	}
	if (photo != nil) {
		[loadedBoardSection setPhotoFilePath:photo];
		[loadedBoardSection setPhotoAnnotation:[UIImage imageWithContentsOfFile:photo]];
		[loadedBoardSection setHasPhoto:YES];
	}
	if (audio != nil)
		[loadedBoardSection setSoundFile:audio];
	
	return loadedBoardSection;
}

Board * loadBoard(NSMutableDictionary * boardDict) {
	Board * loadedBoard = [[Board alloc] initWithLayoutType:(LayoutType)[[boardDict objectForKey:@"layout"] intValue]];
	NSMutableArray *boardSectionDicts = [boardDict objectForKey:@"sections"];
	//NSMutableArray *boardSections = [[NSMutableArray alloc] init];
	
	//[loadedBoard setLayout:(LayoutType)[[boardDict objectForKey:@"layout"] intValue]];
	[loadedBoard setMinutes:[[boardDict objectForKey:@"minutes"] intValue]];
	
	int i = 0;
	for (NSMutableDictionary * boardSectionDict in boardSectionDicts) {
		loadBoardSection(boardSectionDict, [loadedBoard.sections objectAtIndex:i]);
		//[boardSections addObject:loadBoardSection(boardSectionDict)];
		i++;
	}
	//loadedBoard.sections = [[NSArray alloc] initWithArray:boardSections];
	return loadedBoard;
}

NSString * loadLectureName(NSString *lectureIdentifier) {
	NSMutableDictionary *lecture;
	NSString * rootPath = NSHomeDirectory();
	NSString * lectureNamesPath = [[NSString alloc] initWithFormat:@"%@/Documents/%@.plist", rootPath, lectureIdentifier];
	
	if ([[NSFileManager defaultManager] fileExistsAtPath:lectureNamesPath]) {
		NSString *error;
		NSPropertyListFormat format;
		NSData *names = [[NSFileManager defaultManager] contentsAtPath:lectureNamesPath];
		NSDictionary *temp2 = (NSDictionary *)[NSPropertyListSerialization propertyListFromData:names mutabilityOption:NSPropertyListMutableContainersAndLeaves format:&format errorDescription:&error];
		lecture = [[NSMutableDictionary alloc] initWithDictionary:temp2];
	
		return [lecture objectForKey:@"title"];
	}
	return nil;
}

Lecture * loadLecture(NSString *lectureIdentifier) {
	NSMutableDictionary *lecture;
	NSString * rootPath = NSHomeDirectory();
	NSString * lectureNamesPath = [[NSString alloc] initWithFormat:@"%@/Documents/%@.plist", rootPath, lectureIdentifier];

	if ([[NSFileManager defaultManager] fileExistsAtPath:lectureNamesPath]) {
		NSString *error;
		NSPropertyListFormat format;
		NSData *names = [[NSFileManager defaultManager] contentsAtPath:lectureNamesPath];
		NSDictionary *temp2 = (NSDictionary *)[NSPropertyListSerialization propertyListFromData:names mutabilityOption:NSPropertyListMutableContainersAndLeaves format:&format errorDescription:&error];
		lecture = [[NSMutableDictionary alloc] initWithDictionary:temp2];
	
	
		Lecture * loadedLecture = [[Lecture alloc] init];
		[loadedLecture setTitle:[lecture objectForKey:@"title"]];
		[loadedLecture setTiming:(BOOL)[[lecture objectForKey:@"timing"] intValue]];
		[loadedLecture setDuration:[[lecture objectForKey:@"duration"] intValue]];
		// Create Physical boards
		
		NSDictionary *physBoardPosDict = (NSDictionary*) [lecture objectForKey:@"physBoardPos"];
		for (id i in [physBoardPosDict allValues]) {
			NSDictionary *d = (NSDictionary*) i;
			CGPoint origin = CGPointMake([((NSString*)[d objectForKey:@"x"]) floatValue], 
										 [((NSString*) [d objectForKey:@"y"]) floatValue]);
			[loadedLecture.physBoardPos addObject:[NSValue valueWithCGPoint:origin]];
		}
		

		NSNumber * boardCount = [[NSNumber alloc] initWithInt:[[lecture objectForKey:@"numBoards"] intValue]];
		NSMutableDictionary *boardDict;
	
		//Reload all of the boards from memory
		for (int i = 1; i <= [boardCount intValue]; i++) {
			boardDict = [lecture objectForKey:[[NSString alloc] initWithFormat:@"board%d", i]];
			[[loadedLecture boardList] addObject:loadBoard(boardDict)];
		}
	
		return loadedLecture;
	}
	
	return nil;
}

@end
