//
//  LectureSettingsViewController.m
//  WBA
//
//  Created by Class Account on 3/18/10.
//  Copyright 2010 __MyCompanyName__. All rights reserved.
//

#import "LectureSettingsViewController.h"
#import "LectureOverviewViewController.h"
#import "ClassroomConfigViewController.h"
#import "LectureDurationPicker.h"

@implementation LectureSettingsViewController


@synthesize myTableView;
@synthesize parent, durationLabel;
@synthesize model;

/*
 // The designated initializer.  Override if you create the controller programmatically and want to perform customization that is not appropriate for viewDidLoad.
- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
    if (self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil]) {
        // Custom initialization
    }
    return self;
}
*/

-(void)viewWillAppear:(BOOL)animated
{
	[super viewWillAppear:animated];
	
	//titleField.text = model.title;
}

// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad {
    [super viewDidLoad];
	self.navigationItem.title = @"Lecture Settings";
	self.navigationController.navigationBarHidden = NO;
}

-(void)setModel:(Lecture*)newModel
{
	if (model)
		[model release];
	model = newModel;
	[model retain];
}

-(Lecture*)getLecture
{
	return model;
}


-(void) acceptSettings{
	// Set the lecture title
  if ([titleField.text isEqualToString:EMPTY_LECTURE_TITLE]) {
    titleField.text = DEFAULT_LECTURE_TITLE;
  } else {
    model.title = titleField.text;
  }
  [model.title retain];
  [parent settingsAccepted];
}

// Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations
	if (interfaceOrientation == UIInterfaceOrientationLandscapeLeft || interfaceOrientation == UIInterfaceOrientationLandscapeRight){
		return YES;
	}else
	{
		//Doing it this way because we may want some screens to support portrait later, like the lecture chooser.
		return NO;
	}
}

- (void)didReceiveMemoryWarning {
	// Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
	
	// Release any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload {
	// Release any retained subviews of the main view.
	// e.g. self.myOutlet = nil;
}


- (void)dealloc {
	if (model)
		[model release];
	[myTableView release];
	[parent release];
	[titleField release];
	if (durationLabel)
		[durationLabel release];
	[timingSwitch release];
    [super dealloc];
}

# pragma mark textfieldstuff

-(void)textFieldDidBeginEditing: (UITextField*)textField  {
  if (textField == titleField) {
    if ([model.title isEqualToString:EMPTY_LECTURE_TITLE]) {
      textField.text = @"";
    }
  }
}

-(void)textFieldDidEndEditing: (UITextField*)textField  {
  if (textField == titleField) {
    if ([titleField.text isEqualToString:EMPTY_LECTURE_TITLE]) {
      titleField.text = DEFAULT_LECTURE_TITLE;
    } else {
      model.title = titleField.text;
    }
  }
}


-(BOOL)textFieldShouldReturn:(UITextField*)theTextField
{
	[theTextField resignFirstResponder];
	return YES;
}


#pragma mark tableviewstuff

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return 2;
}


// Customize the number of rows in the table view.
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    if (section == 0 && model.timing)
		return 4;
	else if (section == 0)
		return 3;
	else
		return 1;
}

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section
{
	if (section == 0)
		return @"Lecture Settings";
	else return @"";
}

// Customize the appearance of table view cells.
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    
	UITableViewCell *cell = nil;
	
	if (indexPath.section == 0){
		
		static NSString *NewLectureIdentifier = @"NewLectureCell";
		
		cell = [tableView dequeueReusableCellWithIdentifier:NewLectureIdentifier];
		if (cell == nil) {
			cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:NewLectureIdentifier] autorelease];
		}
		
		if (indexPath.row == 0)
		{
			cell.selectionStyle = UITableViewCellSelectionStyleNone;
			cell.textLabel.text = @"Lecture Title:";
			
			// Add text field subview for editing the title
			if (!titleField)
			{
				titleField = [[UITextField alloc] initWithFrame:
							  CGRectMake(cell.frame.origin.x+125, cell.frame.origin.y+11, cell.frame.size.width, cell.frame.size.height-20)];
				titleField.returnKeyType = UIReturnKeyDone;
				titleField.delegate = self;
				
				if ([model.title isEqualToString:EMPTY_LECTURE_TITLE]) {
				  titleField.text = DEFAULT_LECTURE_TITLE;
				} else {
				  titleField.text = model.title;
				}
				titleField.textAlignment = UITextAlignmentRight;
				titleField.autocapitalizationType = UITextAutocapitalizationTypeWords;
				[cell.contentView addSubview:titleField];
			}
		}
		if (indexPath.row == 1)
		{
			cell.textLabel.text = @"Classroom Board Arrangement...";
		}
		if (indexPath.row == 2)
		{
			cell.selectionStyle = UITableViewCellSelectionStyleNone;
			cell.textLabel.text = @"Enable Timing?";
			
			// Add switch
			if (!timingSwitch)
			{
				timingSwitch = [[UISwitch alloc] initWithFrame:
								CGRectMake(cell.frame.origin.x+350, cell.frame.origin.y+8, cell.frame.size.width, cell.frame.size.height-20)];
				timingSwitch.on = model.timing;
				[timingSwitch addTarget:self action:@selector(switchToggled) forControlEvents:UIControlEventValueChanged];
				[cell.contentView addSubview:timingSwitch];
			}
		}
		if (indexPath.row == 3)
		{
			cell.textLabel.text = @"Lecture Duration:";
			
			if (!durationLabel)
			{
				durationLabel = [[UILabel alloc] initWithFrame:
							 CGRectMake(cell.frame.origin.x+250, cell.frame.origin.y+11, cell.frame.size.width-126, cell.frame.size.height-20)];
				durationLabel.textAlignment = UITextAlignmentRight;
				if (model.timing)
				{
					durationLabel.text = [NSString stringWithFormat:@"%d hours, %d mins", model.duration/60, model.duration % 60];
				}
				else durationLabel.enabled = NO;
				[cell.contentView addSubview:durationLabel];
			}
				
		} 
	}
	else{
		static NSString *LectureCellIdentifier = @"LectureCell";
		
		cell = [tableView dequeueReusableCellWithIdentifier:LectureCellIdentifier];
		if (cell == nil) {
			cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:LectureCellIdentifier] autorelease];
		}
		cell.textLabel.textAlignment = UITextAlignmentCenter;
		cell.textLabel.text = @"Done";
	}
	
	
    return cell;
}


- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    // Navigation logic may go here. Create and push another view controller.
	// AnotherViewController *anotherViewController = [[AnotherViewController alloc] initWithNibName:@"AnotherView" bundle:nil];
	// [self.navigationController pushViewController:anotherViewController];
	// [anotherViewController release];
	if (indexPath.section == 0)
	{
		if (indexPath.row == 0)
		{
		  // Call up keyboard to edit title
		}
		if (indexPath.row == 1)
		{
			// Launch view controller for classroom board config
			ClassroomConfigViewController *ccVC = [[ClassroomConfigViewController alloc] initWithNibName:@"ClassroomConfigViewController" bundle:[NSBundle mainBundle]];
			ccVC.lecture = model;
			[self presentModalViewController:ccVC animated:YES];
			[ccVC release];
		}
		if (indexPath.row == 2)
		{
			// Toggle timing
		}
		if (indexPath.row == 3)
		{
			// Call up keyboard to edit duration
			LectureDurationPicker *ldp = [[LectureDurationPicker alloc] initWithNibName:@"LectureDurationPicker" bundle:[NSBundle mainBundle]];
			ldp.lecture = model;
			ldp.lsvc = self;
			[self presentModalViewController:ldp animated:YES];
			[ldp release];
			
		}
	}
	else
	{
		if (indexPath.row == 0)
		{
			[self acceptSettings];
		}
	}
	
	[tableView deselectRowAtIndexPath:indexPath animated:YES];
}

-(void)switchToggled
{
	model.timing = (timingSwitch.on);
	durationLabel.enabled = (timingSwitch.on);
	if (durationLabel.enabled)
	{
		durationLabel.text = [[NSNumber numberWithInt:model.duration] stringValue];
	}
	else
	{
		durationLabel.text = @"";
	}
	
	[myTableView reloadData];
}

-(void)updateDuration {
	NSString *hours;
	NSString *mins;
	if (model.duration / 60 == 1)
		hours = @"hour";
	else
		hours = @"hours";
	if (model.duration % 60 == 1)
		mins = @"min";
	else mins = @"mins";
	durationLabel.text = [NSString stringWithFormat:@"%d %@, %d %@", model.duration/60, hours, model.duration % 60, mins];
}

@end
