//
//  PresentationViewController.m
//  WBA
//
//  Created by Class Account on 4/1/10.
//  Copyright 2010 __MyCompanyName__. All rights reserved.
//

#import "PresentationViewController.h"
#import "PresentationBoardView.h"
#import "HelpViewController.h"
#import "PresentationImageViewController.h"
#import "PhysBoardLocViewController.h"


@implementation PresentationViewController

@synthesize scrollView;
@synthesize myToolBar;
@synthesize boardPos;


 // The designated initializer.  Override if you create the controller programmatically and want to perform customization that is not appropriate for viewDidLoad.
- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
    if (self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil]) {
        // Custom initialization
		boardViews = [[NSMutableArray alloc] init];
		timeElapsed = 0;
    }
    return self;
}

/*
// Implement loadView to create a view hierarchy programmatically, without using a nib.
- (void)loadView {
}
*/


// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad {
	
	// Not ready to load yet...
	if (!lecture)
		return;
	
    [super viewDidLoad];
	self.navigationController.navigationBarHidden = YES;
	
	// Set up scroll view
	int n = [lecture numBoards];
	[self.scrollView setContentSize:CGSizeMake(n*scrollView.frame.size.width, scrollView.frame.size.height)];
	
	// Set up board views
	for (int i = 0; i < n; i++)
	{	
		// First, set up arrows
		if (i > 0)
		{
			UIButton *arrow_back = [UIButton buttonWithType:UIButtonTypeCustom];
			arrow_back.frame = CGRectMake(12+i*(PRESMODE_PADDING+PRESMODE_BOARD_WIDTH), 2+PRESMODE_BOARD_Y+PRESMODE_BOARD_HEIGHT/2-15, 30, 30);
			arrow_back.enabled = NO;
			[arrow_back setBackgroundImage:[UIImage imageNamed:@"arrow_back.png"] forState:UIControlStateDisabled];
			[self.scrollView addSubview:arrow_back];
		}
		if (i < n - 1)
		{
			UIButton *arrow_forward = [UIButton buttonWithType:UIButtonTypeCustom];
			arrow_forward.frame = CGRectMake(PRESMODE_PADDING/2+PRESMODE_BOARD_X+PRESMODE_BOARD_WIDTH+10+i*(PRESMODE_PADDING+PRESMODE_BOARD_WIDTH), 2+PRESMODE_BOARD_Y+PRESMODE_BOARD_HEIGHT/2-15, 30, 30);
			arrow_forward.enabled = NO;
			[arrow_forward setBackgroundImage:[UIImage imageNamed:@"arrow_forward.png"] forState:UIControlStateDisabled];
			[self.scrollView addSubview:arrow_forward];
		}
		
		// Then, set up "#/total" labels
		CGRect labelframe = CGRectMake(PRESMODE_PADDING/2+PRESMODE_BOARD_X+PRESMODE_BOARD_WIDTH+14+i*(PRESMODE_PADDING+PRESMODE_BOARD_WIDTH), 0, 40, 40);
		UILabel *numLabel = [[UILabel alloc] initWithFrame:labelframe];
		numLabel.text = [NSString stringWithFormat:@"%d/%d", i+1, [lecture numBoards]];
		numLabel.backgroundColor = [numLabel.backgroundColor colorWithAlphaComponent:0.0];
		[self.scrollView addSubview:numLabel];
		
		CGRect frame = CGRectMake(PRESMODE_PADDING/2+PRESMODE_BOARD_X+i*(PRESMODE_PADDING+PRESMODE_BOARD_WIDTH), 2+PRESMODE_BOARD_Y, PRESMODE_BOARD_WIDTH, PRESMODE_BOARD_HEIGHT);
		PresentationBoardView *pbv = [[PresentationBoardView alloc] initWithFrame:frame];
		[pbv setModel:[lecture getBoard:i]];
		[pbv setViewController:self];
		[self.scrollView addSubview:pbv];
		[boardViews addObject:pbv];
		[pbv release];
	}
	
	if (lecture.timing && [lecture numBoards] > 0 && !timerNotice)
	{
		// Initialize and add timer notice
		timerNotice = [[TimerNoticeView alloc] initWithFrame:CGRectMake(0, 0, 200, 30)];
		NSMutableArray *tmp = [NSMutableArray arrayWithArray:myToolBar.items];
		[tmp insertObject:[[UIBarButtonItem alloc] initWithCustomView:timerNotice] atIndex:2];
		[myToolBar setItems:tmp animated:NO];
		
		// Initialize timer
		myTimer = [NSTimer scheduledTimerWithTimeInterval:TIME_INTERVAL target:self selector:@selector(fireTimer) userInfo:nil repeats:YES]; 
		[myTimer retain];
		timeElapsed = -TIME_INTERVAL;
		[myTimer fire];
	}
}

-(void)viewDidUnload
{
	for (id obj in boardViews)
		[obj release];
	[boardViews removeAllObjects];
	if (timerNotice)
	{
		[timerNotice release];
		timerNotice = nil;
	}
}

-(void)viewDidAppear:(BOOL)animated
{
	[super viewDidAppear:animated];
	[scrollView flashScrollIndicators];
}

-(void)viewWillAppear:(BOOL)animated
{
	[super viewWillAppear:animated];
	
	if (lecture.physBoardPos.count == 0 || [lecture numBoards] == 0)
		boardPos.enabled = NO;
	
	// Stupid trick to get timerNotice to reappear when returning from image viewer
	NSMutableArray *tmp = [NSMutableArray arrayWithArray:[myToolBar items]];
	[myToolBar setItems:tmp animated:NO];
}

-(void)fireTimer
{
	timeElapsed += TIME_INTERVAL;
	
	[self updateTimerNotice];
}
-(void)updateTimerNotice
{	
	if ([lecture numBoards] > 0)
	{
		int timeUpToEndOfBoard = 0;
		for (int i = 0; i <= currBoard; i++)
			timeUpToEndOfBoard += [lecture getBoard:i].minutes*60;
		int timeLeftForBoard = MAX(timeUpToEndOfBoard - timeElapsed, 0);
		
		NSLog(@"timeElapsed: %d", timeElapsed);
		NSLog(@"timeLeftForBoard: %d", timeLeftForBoard);
		
		// Notify the timer notice view
		[timerNotice update:lecture.duration elapsed:timeElapsed forBoard:timeLeftForBoard];
	}
}


-(void)setLecture:(Lecture*)lec
{
	if (lecture)
		[lecture release];
	lecture = lec;
	[lecture retain];
}

-(IBAction)viewHelp:(id)sender
{
	HelpViewController *hVC = [[HelpViewController alloc] initWithNibName:@"HelpViewController" bundle:[NSBundle mainBundle]];
	hVC.parentController = self;
	[self presentModalViewController:hVC animated:YES];
	[hVC release];
}

-(IBAction)viewBoardPos
{
	PhysBoardLocViewController *pblVC = [[PhysBoardLocViewController alloc] initWithNibName:@"PhysBoardLocViewController" bundle:[NSBundle mainBundle]];
	pblVC.lecture = lecture;
	pblVC.board = [lecture getBoard:currBoard];
	pblVC.isEditing = NO;
	[self presentModalViewController:pblVC animated:YES];
	[pblVC release];
}

-(void)launchPhotoViewer:(UIImage*)img
{
	PresentationImageViewController *piVC = [[PresentationImageViewController alloc] initWithNibName:@"PresentationImageViewController" bundle:nil];
	[piVC setImage:img];
	[piVC setTimer:timerNotice];
	[self presentModalViewController:piVC animated:NO];
	[piVC release];
}

-(IBAction)exitPresentation:(id)sender
{
	[myTimer invalidate];
	
	// Giant hack to avoid the problem of the lecture overview scrolling
	// vertically upon returning from presentation mode
	[[self.navigationController.viewControllers objectAtIndex:0] hackyPopToSelfAndReloadLecture:lecture];
}

// Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations
	if (interfaceOrientation == UIInterfaceOrientationLandscapeLeft || interfaceOrientation == UIInterfaceOrientationLandscapeRight){
		return YES;
	}else
	{
		//Doing it this way because we may want some screens to support portrait later, like the lecture chooser.
		return NO;
	}
}

- (void)didReceiveMemoryWarning {
	// Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
	
	// Release any cached data, images, etc that aren't in use.
}


- (void)scrollViewDidEndDecelerating:(UIScrollView *)scrollView
{
	float screenleft = scrollView.contentOffset.x;
	
	currBoard = (int)(screenleft / (PRESMODE_BOARD_WIDTH+PRESMODE_PADDING));
	NSLog(@"boardnum: %d", currBoard);
	
	NSLog(@"numtoolbaritems: %d", myToolBar.items.count);
	
	// let the timer notice know that the board has changed
	[self updateTimerNotice];
}


- (void)dealloc {
	if (lecture)
		[lecture release];
	[boardViews release];
	[myTimer release];
	[boardPos release];
    [super dealloc];
}


@end
