//
//  LecturesViewController.m
//  WBA
//
//  Created by Class Account on 3/18/10.
//  Copyright 2010 __MyCompanyName__. All rights reserved.
//

#import "LecturesViewController.h"
#import "LectureSettingsViewController.h"
#import "LectureOverviewViewController.h"
#import "WBAAppDelegate.h"


@implementation LecturesViewController

@synthesize tableView;
@synthesize currentLecture;
@synthesize lecturesDataList;

#pragma mark Regular Methods

 // The designated initializer.  Override if you create the controller programmatically and want to perform customization that is not appropriate for viewDidLoad.
- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
    if (self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil]) {
        // Custom initialization
		lectures = [[NSMutableArray alloc] init];
		masterStateFilePath = [[[NSString alloc] initWithFormat:@"%@/Documents/masterState.plist", NSHomeDirectory()] retain];
		resourcesDir = [[[NSString alloc] initWithFormat:@"%@/Documents/", NSHomeDirectory()] retain];
		if (! [self loadStateFromDisk]) {
		  // If we fail while loading our state, then create a new lecturesDataList
		  lecturesDataList = [[NSMutableArray alloc] init];
		}
		[lecturesDataList retain];
    }
    return self;
}

-(id)initWithCoder:(NSCoder*)coder
{
	if (self = [super initWithCoder:coder])
	{
		// Custom initialization
		lectures = [[NSMutableArray alloc] init];
		masterStateFilePath = [[[NSString alloc] initWithFormat:@"%@/Documents/masterState.plist", NSHomeDirectory()] retain];
		resourcesDir = [[[NSString alloc] initWithFormat:@"%@/Documents/", NSHomeDirectory()] retain];
		if (! [self loadStateFromDisk]) {
		  // If we fail while loading our state, then create a new lecturesDataList
		  lecturesDataList = [[NSMutableArray alloc] init];
		}
		[lecturesDataList retain];
	}
	return self;
}

-(void)viewWillAppear:(BOOL)animated
{
	[super viewWillAppear:animated];

	WBAAppDelegate *ad = [[UIApplication sharedApplication] delegate];
	ad.lecturesVC = (UIViewController*) self;
	
	// This is a sort of hack....
	[self serializeCurrentLecture];
	
	// Set current lecture to nil so that we don't serialize it after deleting it.
	currentLecture = nil;
	
	// Tell table view to reload contents
	[tableView reloadData];
	[self validateEditing];
}


- (BOOL) loadStateFromDisk {
  if ([[NSFileManager defaultManager] fileExistsAtPath:masterStateFilePath]) {
    NSLog(@"Found master state file path at %@", masterStateFilePath);
    NSData *masterStateData = [[NSFileManager defaultManager] contentsAtPath:masterStateFilePath];

    NSPropertyListFormat format;
    NSString *error = nil;
    lecturesDataList = (NSMutableArray *)
      [NSPropertyListSerialization propertyListFromData: masterStateData
				   mutabilityOption: NSPropertyListMutableContainersAndLeaves 
				   format:&format 
				   errorDescription:&error];
    [lecturesDataList retain];
    if (lecturesDataList == nil || error != nil) {
      NSLog(@"Failed at turning masterStateData into an MSMutableArray, this is the error:\n\t%@",
	    error);
      [error release];
    }
    return (lecturesDataList != nil);
  } else {
    NSLog(@"Can not find master state file path at %@", masterStateFilePath);
    return NO;
  }
}

- (void) serializeCurrentLecture {
  // Takes the Lecture *currentLecture, creates its NSDictionary
  // representation, and writes it in the lecturesDataList,
  // overwriting an stale version if possible

  int lectureIndex = 0;
  int i = 0;
  BOOL isAlreadyInList = NO;

  if (currentLecture != nil) {
    NSMutableDictionary *d = [currentLecture serialize];
    [d retain];


    for (NSMutableDictionary *d in lecturesDataList) {
      if ([currentLecture.lid isEqualToString: (NSString*) [d objectForKey:LEC_ID_KEY]]) {
	NSLog(@"They are equal");
	lectureIndex = i;
	isAlreadyInList = YES;
	break;
      }
      i += 1;
    }

    // MRU behavior
    if (isAlreadyInList) {
      //[lecturesDataList removeObjectAtIndex:lectureIndex];
		[lecturesDataList replaceObjectAtIndex:lectureIndex	withObject:d];
    }
    else [lecturesDataList insertObject:d atIndex: 0];
  }

}


- (void) serializeStateToDisk {
  // lecturesDataList is an array of NSMutableDictionaries. This is
  // the serialized representation of our lectures. The current
  // lecture is NOT in the lecturesDataList. If we want to save the
  // current lecture (which we do), we need to serialize it and push
  // it into lecturesDataList. This is why serializeStateToDisk calls
  // [self maybeUnloadCurrentLecture]. Only NSDictionaries and its
  // subclasses can be written to disk, and we definitely need
  // currentLecture to be written out.
  [self serializeCurrentLecture];
  if(! [lecturesDataList writeToFile:masterStateFilePath atomically:YES]) {
    NSLog(@"Failed at writing to state to file %@", masterStateFilePath);
  }
}


- (void) removeStaleResources {
  // Do a simple search to find all the resources that are reachable
  [self registerReferencedResources];
  

  // Iterate across all of our files and delete all files that are not
  // in the list of reachable files
  NSFileManager *fm = [[NSFileManager alloc] init];
  NSDirectoryEnumerator *dirEnum = [fm enumeratorAtPath:resourcesDir];
  NSString *rsrc;
  
  while(rsrc = [dirEnum nextObject]) {
    NSLog(@"Have a resource %@", rsrc);
    NSString *fullPath = [[NSString alloc] initWithFormat:@"%@/%@", resourcesDir, rsrc];
    if ([referencedResources containsObject:fullPath]) {
      NSLog(@"     This resource has been seen. Keeping.");
    } else {
      NSLog(@"     This resource will be unlinked");
      if (![fm removeItemAtPath:fullPath error:nil]) {
	NSLog(@"     Error at unlinking resource")  ;
      }
    }
  }
}

- (void) registerReferencedResources {
  // If there exists a lecture that uses a resource R as an
  // annotation, then R is reachable and should not be deleted.
  
  referencedResources = [[NSMutableSet alloc] init];
	[referencedResources addObject:masterStateFilePath];
  for (NSMutableDictionary *l in lecturesDataList) {
    for (NSMutableDictionary *b in [l objectForKey:LEC_BOARDS_KEY]) {
      for (NSMutableDictionary *bs in [b objectForKey:BOARD_SECTIONS_KEY]) {
	NSString *bs_sound_resource = [bs objectForKey:BSECT_SOUND_FILE_KEY];
	NSString *bs_photo_resource = [bs objectForKey:BSECT_PHOTO_FILE_KEY];
	if (bs_sound_resource) {
	  [referencedResources addObject:bs_sound_resource];
	}
	if (bs_photo_resource) {
	  [referencedResources addObject:bs_photo_resource];
	}
      }
    }
  }
}



// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad {
    [super viewDidLoad];
	[self.navigationItem setTitle:@"QuickBoards"];
	self.navigationController.navigationBarHidden = NO;
	//[self editButtonItem].enabled = NO;
	
	// Table view editing stuff
	self.navigationItem.rightBarButtonItem = self.editButtonItem;
	self.editButtonItem.action = @selector(editButtonPressed);

	logo = [[UIImageView alloc] initWithImage:[UIImage imageNamed:@"quickboardslogoplainsmall.png"]];
	logo.contentMode = UIViewContentModeScaleAspectFit;
	logo.frame = CGRectMake(0, 0, 300, 35);
	[self.navigationItem setTitleView:logo];

}

-(void)editButtonPressed
{
	[self setEditing:!tableView.editing animated:YES];
	[tableView setEditing:!tableView.editing animated:YES];
	[self validateEditing];
}

-(void)hackyPopToSelfAndReloadLecture:(Lecture*)lec
{
	[self.navigationController popToViewController:self animated:NO];
	LectureOverviewViewController *loVC = [[LectureOverviewViewController alloc] initWithNibName:@"LectureOverviewViewController" bundle:nil];
	[loVC setModel:lec];
	[self.navigationController pushViewController:loVC animated:NO];
	[loVC release];
}


// Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations
	if (interfaceOrientation == UIInterfaceOrientationLandscapeLeft || interfaceOrientation == UIInterfaceOrientationLandscapeRight){
		return YES;
	}else
	{
		//Doing it this way because we may want some screens to support portrait later, like the lecture chooser.
		return NO;
	}
}


- (void)didReceiveMemoryWarning {
	// Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
	
	// Release any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload {
	// Release any retained subviews of the main view.
	// e.g. self.myOutlet = nil;
}

-(void)settingsAccepted
{
        Lecture *lec = [(LectureSettingsViewController*)(self.modalViewController) getLecture];
	[self dismissModalViewControllerAnimated:NO];
	[self.modalViewController release];
	
	
	LectureOverviewViewController *loVC = [[LectureOverviewViewController alloc] initWithNibName:@"LectureOverviewViewController" bundle:nil];
	[loVC setModel:lec];
	[self.navigationController pushViewController:loVC animated:YES];
	[loVC release];
}



- (void)dealloc {
//	[appDelegate release];
	[lectures release];
    [super dealloc];
}

#pragma mark Table view methods

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return 2;
}


// Customize the number of rows in the table view.
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    if (section == 0)
		return 1;
	else
		return [lecturesDataList count];
}

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section
{
	if (section == 0)
		return @"Create Lecture";
	else if (section == 1 && [lecturesDataList count] > 0)
		return @"Load Lecture";
	else return @"";
}

// Customize the appearance of table view cells.
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    
	UITableViewCell *cell = nil;
	
	if (indexPath.section == 0){
		
		static NSString *NewLectureIdentifier = @"NewLectureCell";
		
		cell = [tableView dequeueReusableCellWithIdentifier:NewLectureIdentifier];
		if (cell == nil) {
			cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:NewLectureIdentifier] autorelease];
		}
			
		cell.imageView.image = [UIImage imageNamed:@"add.png"];
		cell.textLabel.text = @"Create New Lecture";	
	} else{
		static NSString *LectureCellIdentifier = @"LectureCell";
    
		cell = [tableView dequeueReusableCellWithIdentifier:LectureCellIdentifier];
		if (cell == nil) {
			cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:LectureCellIdentifier] autorelease];
		}
		
		NSDictionary *d = [lecturesDataList objectAtIndex:indexPath.row];
		cell.textLabel.text = [[d objectForKey:LEC_TITLE_KEY] retain];
	}

	
    return cell;
}


- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    // Navigation logic may go here. Create and push another view controller.
	// AnotherViewController *anotherViewController = [[AnotherViewController alloc] initWithNibName:@"AnotherView" bundle:nil];
	// [self.navigationController pushViewController:anotherViewController];
	// [anotherViewController release];
	if (indexPath.section == 0)
	{
		// Create new lecture
		Lecture *newLecture = [[[Lecture alloc] init] retain];
		
		currentLecture = newLecture;

		LectureSettingsViewController *lsVC = [[LectureSettingsViewController alloc]
							initWithNibName:@"LectureSettingsViewController" 
							bundle:nil];

		[lsVC setModel:newLecture];
		lsVC.parent = self;
		[self presentModalViewController:lsVC animated:YES];
		[newLecture release];
	}
	else
	{
                // Load existing lecture When we instantiate a Lecture
	        // data structure from lecturesDataList, we need to
	        // remove it from the lecturesDataList.  See comment
	        // in serializeStateToDisk.

		NSDictionary *d = [lecturesDataList objectAtIndex:indexPath.row];
		Lecture *l = [[[Lecture alloc] initWithDict: d] retain];

		currentLecture = l;
		
		LectureOverviewViewController *loVC = [[LectureOverviewViewController alloc] 
							initWithNibName:@"LectureOverviewViewController" 
							bundle:nil];
		[loVC setModel:l];
		[self.navigationController pushViewController:loVC animated:YES];
		[loVC release];
	}
	
	[tableView deselectRowAtIndexPath:indexPath animated:YES];
}

 // Override to support conditional editing of the table view.
 - (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath
{
	 // Return NO if you do not want the specified item to be editable.
	if (indexPath.section != 0)
	 return YES;
	else return NO;
}

-(UITableViewCellEditingStyle)tableView:(UITableView*)tableView editingStyleForRowAtIndexPath:(NSIndexPath*)indexPath
{
	if (indexPath.section != 0)
		return UITableViewCellEditingStyleDelete;
	else return UITableViewCellEditingStyleNone;
}


 // Override to support editing the table view.
 - (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath {
 
	 if (editingStyle == UITableViewCellEditingStyleDelete) {
		 // Delete the row from the data source

	   [lecturesDataList removeObjectAtIndex: indexPath.row];
	   [tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:indexPath] withRowAnimation:YES];
	 }    
	 [tableView reloadData];
 }

-(void)validateEditing{
	// This is just troublesome...
	//[self editButtonItem].enabled = ([lecturesDataList count] > 0);
	[self editButtonItem].enabled = YES;
}




 // Override to support rearranging the table view.
 - (void)tableView:(UITableView *)tableView moveRowAtIndexPath:(NSIndexPath *)fromIndexPath toIndexPath:(NSIndexPath *)toIndexPath {
	 id obj = [lecturesDataList objectAtIndex:fromIndexPath.row];
	 [lecturesDataList removeObjectAtIndex:fromIndexPath.row];
	 [lecturesDataList insertObject:obj	atIndex:toIndexPath.row];
 }



 // Override to support conditional rearranging of the table view.
 - (BOOL)tableView:(UITableView *)tableView canMoveRowAtIndexPath:(NSIndexPath *)indexPath {
 // Return NO if you do not want the item to be re-orderable.
	 if (indexPath.section == 0)
		 return NO;
	 return YES;
 }



@end
